const util = require('util');
const axios = require('axios');
const express = require('express');
const cors = require('cors');
const { ApolloServer } = require('apollo-server-express');
const { up } = require('migrate-mongo');
const helmet = require('helmet');
const compression = require('compression');
const http = require('http');
const config = require('config');
const mongoose = require('mongoose');
const ms = require('ms');
const resolvers = require('./graphql/resolvers');
const typeDefs = require('./graphql/schema');
const getLogger = require('./utils/logger/logger');
const initAgenda = require('./utils/agenda');
const {
    API_ROOT, DB_CONNECTION_STRING, CLOUD_TIERING_NAMESPACE
} = require('./utils/consts');
const saasRegistrationRouter = require('./routes/saas-registration');
const statisticsRouter = require('./routes/statistics');
const healthRouter = require('./routes/health');
const reactRouterFallbackMiddleware = require('./middlewares/react-router-fallback');
const continuationLocalStorageMiddleware = require('./middlewares/continuation-local-storage');
const findAccountAndUpdateContinuationToken = require('./middlewares/get-account-middleware');
const authenticationMiddleware = require('./middlewares/authentication-middleware');
const { createTenancyServiceToken, createOccmServiceToken } = require('./lib/auth0');
const authenticate = require('./operations/authentication-operations');
const { findAccount } = require('./operations/multi-tenancy-operation');

// override default break length which is 80
util.inspect.defaultOptions.breakLength = Infinity;

const app = express();
const logger = getLogger();
const port = config.get('app-port');

process.on('unhandledRejection', (reason, p) => {
    // p is the promise that was rejected
    // reason is the reason the promise was rejected
    logger.info('Unhandled Rejection at:', p, 'reason:', reason);
});

process.on('uncaughtException', err => logger.info('Uncaught exception was thrown', err));

process.on('SIGINT', signal => {
    logger.info(`Received signal ${signal}`);
    process.exit(0);
});

async function connectToMongo() {
    try {
        logger.info('Connecting to DB:', DB_CONNECTION_STRING);
        await mongoose.connect(DB_CONNECTION_STRING, {
            useNewUrlParser: true,
            dbName: 'cloudTiering',
            useCreateIndex: true
        });
        logger.info('Connected to db');
        const db = mongoose.connection;

        // migrate
        const migrated = await up(db);
        migrated.forEach(fileName => logger.info('Migrated:', fileName));

        return db;
    } catch (err) {
        logger.error('Failed connecting to mongo', err);
        process.exit(-1);
    }
}

async function startServer() {
    const httpServer = http.createServer(app);

    const db = await connectToMongo();

    // configure security helmet
    app.use(helmet());
    app.use(compression());
    app.use(cors({
        origin: (origin, callback) => {
            const allowedDomains = [/netapp\.com/, /netapp\.io/, /cloudtiering\.io/];

            if (allowedDomains.some(domain => domain.test(origin))) {
                callback(null, true);
            } else {
                callback(null, false);
            }
        }
    }));

    // configure static assets middleware
    const staticAssetsMiddleware = express.static(config.get('paths.ui'));
    app.use(staticAssetsMiddleware);

    // always return index.html in case no resource found and it's not an API call
    app.use(reactRouterFallbackMiddleware(staticAssetsMiddleware));

    app.use(express.json());// json body-parser
    app.use(express.urlencoded({ extended: true }));// urlencoded body-parser qs module instead of query-string- assumption we will need a deep object

    app.use(continuationLocalStorageMiddleware);

    app.use(API_ROOT, healthRouter);

    if (process.env.NODE_ENV !== 'production') {
        app.all(/api\/test.*/, async (req, res, next) => {
            const { headers, method } = req;

            const { authorization } = headers;

            try {
                if (!headers.forwardurl) {
                    const { data } = await axios.post('https://127.0.0.1:8888/simulator/change-scenario', req.body, { headers: { authorization: authorization || '' } });
                    return res.json(data);
                }
                if (method.toLowerCase() === 'delete') {
                    const { data } = await axios[method.toLowerCase()](headers.forwardurl, { headers: { authorization } });
                    return res.json(data);
                }
                const { data } = await axios[method.toLowerCase()](headers.forwardurl, req.body, { headers: { authorization } });
                res.json(data);
            } catch (err) {
                next(err);
            }
        });
    }

    await Promise.all([createTenancyServiceToken(), createOccmServiceToken()]);
    await initAgenda(db);

    app.use(authenticationMiddleware);
    app.use(findAccountAndUpdateContinuationToken);

    if (process.env.IS_SIMULATOR) {
        app.use(API_ROOT, require('./routes/simulator')); // eslint-disable-line global-require
    }

    app.use(API_ROOT, statisticsRouter);
    app.use(API_ROOT, saasRegistrationRouter);

    const apollo = new ApolloServer({
        typeDefs,
        resolvers,
        formatError: (error => {
            if (error.extensions.code === 'INTERNAL_SERVER_ERROR') {
                error.extensions.code = error.extensions.exception.status;
                error.extensions.message = error.message;
            }
            logger.error('Received Error ', JSON.stringify(error));
            return error;
        }),
        context: (async ({ req }) => {
            if (!(req.body && req.body.operationName)) {
                const authorizationToken = req.headers.Authorization || req.headers.authorization || '';
                await authenticate(authorizationToken);
                const accountId = await findAccount();
                CLOUD_TIERING_NAMESPACE.set('accountId', accountId);
                return { accountId };
            }
        }),
        mocks: false, // for mocks responses until the simulator will work
        introspection: true, // for playground in production mode
        playground: {
            settings: {
                'editor.theme': 'dark',
                'editor.reuseHeaders': true
            },
            tabs: [{
                endpoint: '/graphql',
                query: '{clusters { clusterId }}',
                name: 'query clusters',
                headers: { authorization: 'user-token' }
            }]
        },
        extensions: [() => ({
            requestDidStart: ({ queryString, variables }) => logger.info('GraphQL request', JSON.stringify({ queryString: queryString.replace(/(\r\n|\n|\r)/gm, ''), variables })),
            willSendResponse: ({ graphqlResponse }) => logger.info('GraphQL response', JSON.stringify(graphqlResponse))
        })]
    });

    apollo.applyMiddleware({ app });

    httpServer.listen({ port }, () => {
        logger.info('🚀 Server ready at',
            `http://local.cloudtiering.io:${port}${apollo.graphqlPath}`);
    });

    httpServer.setTimeout(ms('5m'));
}

startServer();
