const Joi = require('joi');
const {
    DISCOVER_LOCATION, AUTO, NONE, SNAPSHOT_ONLY
} = require('../utils/consts');

/** *******************************************
 *          Response validations             *
 ******************************************** */

const capacitySchema = Joi.object().keys({
    size: Joi.number().required(),
    unit: Joi.string().required()
});

const objectStoreConfigSchema = Joi.object().keys({
    objectStoreName: Joi.string().required(),
    s3Name: Joi.string().required(),
    usedSpace: capacitySchema.required(),
    providerType: Joi.string().required(),
    server: Joi.string().required()
});

const capacityFeaturesSchema = Joi.object().keys({
    objectStoreConfigExtended: Joi.object().keys({
        objectStoreConfig: objectStoreConfigSchema.allow(null),
        tierLevel: Joi.string().required().valid('normal', 'ia', 'hot').allow(null)
    }).allow(null),
    performanceTierUsedCapacity: capacitySchema,
    inactiveUserData: capacitySchema.allow(null),
    rawCapacity: capacitySchema
});

const lifSchema = Joi.object().keys({
    name: Joi.string(),
    role: Joi.string().valid('undef', 'data', 'node_mgmt', 'cluster_mgmt', 'cluster', 'intercluster'),
    status: Joi.string().valid('up', 'down'),
    ipSpace: Joi.string().allow('')
});

const clusterResponseSchema = Joi.object().keys({
    name: Joi.string().required(),
    publicId: Joi.string().required(),
    ontapClusterProperties: Joi.object().keys({
        clusterUuid: Joi.string().required(),
        ontapVersion: Joi.string().required().allow(''),
        clusterAddress: Joi.string().required(),
        nodeModels: Joi.array().items(Joi.string()),
        lifs: Joi.array().items(lifSchema)
    }).allow(null),
    capacityFeatures: capacityFeaturesSchema.allow(null),
    status: Joi.object().keys({
        status: Joi.string().required().valid('ON', 'FAILED'),
        message: Joi.string().allow('', null),
        failureCauses: Joi.object().keys({
            invalidOntapCredentials: Joi.boolean()
        }).allow(null),
        extendedFailureReason: Joi.string().allow(null)
    }).allow(null)
});


const clusterMetadataResponseSchema = Joi.object().keys({
    clusterName: Joi.string().required(),
    ontapVersion: Joi.string().required(),
    nodeModels: Joi.array().items(Joi.string())
});

const tieringRestrictionSchema = Joi.object().keys({
    isTieringRestricted: Joi.boolean().required(),
    restrictionReason: Joi.string().allow(null)
});

const aggregateResponseSchema = Joi.object().keys({
    name: Joi.string().required(),
    state: Joi.string().required(),
    objectStoreName: Joi.string().allow(null),
    isObjectStoreAttachEligible: Joi.boolean().required(),
    isInactiveDataReportingEnabled: Joi.boolean().required(),
    usedCapacity: capacitySchema.required(),
    tieringRestriction: tieringRestrictionSchema.required()
});

const aggregatesResponseSchema = Joi.array().items(aggregateResponseSchema);

const volumeSchema = Joi.object().keys({
    name: Joi.string().required(),
    svmName: Joi.string().required(),
    aggregateName: Joi.string().allow(''),
    size: capacitySchema.required(),
    usedSize: capacitySchema.required(),
    tierInactiveUserData: capacitySchema.allow(null),
    tierInactiveUserDataPercent: Joi.number().allow(null),
    tieringPolicy: Joi.string().allow(AUTO, NONE, SNAPSHOT_ONLY, 'backup', null),
    capacityTierUsedSize: capacitySchema.allow(null),
    state: Joi.string().allow('mixed', 'offline', 'restricted', 'online', null),
    constituentsAggregates: Joi.array().required().items(Joi.string())
});

const volumesResponseSchema = Joi.array().items(volumeSchema);

const licenseStatusInformationResponseSchema = Joi.object().keys({
    currentCapacity: capacitySchema.required(),
    maxCapacity: capacitySchema.required(),
    serialNumber: Joi.string().required()
});

const licenseStatusResponseSchema = Joi.object().keys({
    licenseInformation: licenseStatusInformationResponseSchema.allow(null)
});

/** *******************************************
*          Request validations               *
********************************************* */
const discoverRequestSchema = Joi.object().keys({
    tenantId: Joi.string().required(),
    name: Joi.string().required(),
    clusterAddress: Joi.string().ip().required(),
    clusterUserName: Joi.string().required(),
    clusterPassword: Joi.string().required(),
    location: Joi.string().required().valid(DISCOVER_LOCATION)
});

const associatedSubscriptionResponse = Joi.array().items(Joi.object().keys({
    subscriptionId: Joi.string().required(),
    name: Joi.string().required(),
    default: Joi.boolean().required()
}));

const azureRegionsResponse = Joi.array().items(Joi.object().keys({
    displayName: Joi.string().required(),
    name: Joi.string().required()
}));

const awsKeysSchema = Joi.object().keys({
    accessKey: Joi.string().required(),
    secretKey: Joi.string().required()
});

const configureFabricPoolResourcesRequest = Joi.object().keys({
    region: Joi.string().required(),
    awsAccessKeys: awsKeysSchema.required(),
    tierLevel: Joi.string().required().valid('normal', 'ia'),
    ipSpace: Joi.string()
});

const configureAzureFabricPoolResourcesRequest = Joi.object().keys({
    region: Joi.string().required(),
    ipSpace: Joi.string()
});

const licenseSchema = Joi.object().keys({
    statusResp: Joi.object().keys({
        serialNumber: Joi.string().required(),
        version: Joi.string().required(),
        product: Joi.string().required(),
        licenses: Joi.object().keys({
            capacity: Joi.number().required(),
            type: Joi.string().required(),
            package: Joi.string().required()
        })
    }),
    trackingId: Joi.string().required(),
    Signature: Joi.string().required()
});

const modifyVolumeRequestSchema = Joi.object().keys({
    tieringPolicy: Joi.string().required().valid('auto', 'none', 'snapshot_only')
});

const objectStoreAttachRequestSchema = Joi.object().keys({
    workingEnvironmentId: Joi.string(),
    objectStoreName: Joi.string(),
    aggregateName: Joi.string().required(),
    checkOnly: Joi.boolean().required()
});

module.exports = {
    clusterResponseSchema,
    clusterMetadataResponseSchema,
    aggregatesResponseSchema,
    volumesResponseSchema,
    licenseStatusResponseSchema,
    discoverRequestSchema,
    licenseSchema,
    modifyVolumeRequestSchema,
    aggregateResponseSchema,
    configureFabricPoolResourcesRequest,
    volumeSchema,
    objectStoreAttachRequestSchema,
    associatedSubscriptionResponse,
    azureRegionsResponse,
    configureAzureFabricPoolResourcesRequest
};
