const { get, compact } = require('lodash');
const Account = require('../models/account');
const { getAllClustersInAccountInternal } = require('./multi-tenancy-operation');
const { listAllSubscriptionsForService } = require('../lib/subscription-service');
const {
    UNSUBSCRIBE_PENDING, SUBSCRIBED, CHARGING_DIMENSION
} = require('../utils/consts');
const capacityToBytes = require('../utils/utils');
const charge = require('../lib/charging-service');

const getLogger = require('../utils/logger/logger');

const logger = getLogger();
const chargingLogger = getLogger('charging');

async function findAccountsWithoutFreeTrial() {
    return Account.find({ freeTrialExpiryDate: { $lt: new Date() } }).exec();
}

/* creates an object {accountId => subscription} */
function createAccountToSubscriptionMap(cloudTieringSubscriptions, candidateAccounts) {
    const subscriptionsToCharge = cloudTieringSubscriptions.filter(subscription => [UNSUBSCRIBE_PENDING, SUBSCRIBED].includes(subscription.subscriptionStatus));

    return subscriptionsToCharge.reduce((accumulator, subscription) => {
        const {
            accounts, subscriptionId: newSubscriptionId, subscriptionStatus: newSubscriptionStatus, createdAt: newCreatedAt
        } = subscription;
        accounts.forEach(account => {
            if (accumulator[account] || candidateAccounts.some(candidate => candidate.accountId === account)) {
                const subscriptionOfAccount = accumulator[account];
                if (subscriptionOfAccount) {
                    const { createdAt: oldCreatedAt, subscriptionStatus: oldStatus } = subscriptionOfAccount;
                    // change if one of the followings:
                    // #1 new subscription was created before old and old status === unsubscribed_pending or  new subscription is subscribed
                    // #2 old status is unsubscribed-pending and the new status is subscribed
                    if ((newCreatedAt < oldCreatedAt && (oldStatus === UNSUBSCRIBE_PENDING || newSubscriptionStatus === SUBSCRIBED))
                        || (oldStatus === UNSUBSCRIBE_PENDING && newSubscriptionStatus === SUBSCRIBED)) {
                        accumulator[account] = {
                            subscriptionId: newSubscriptionId,
                            subscriptionStatus: newSubscriptionStatus,
                            createdAt: newCreatedAt
                        };
                    }
                } else {
                    accumulator[account] = {
                        subscriptionId: newSubscriptionId,
                        subscriptionStatus: newSubscriptionStatus,
                        createdAt: newCreatedAt
                    };
                }
            }
        });
        return accumulator;
    }, {});
}


async function findTieringUsagePerAccount(accountId) {
    const clusters = await getAllClustersInAccountInternal(accountId);
    logger.info('Charging clusters received', JSON.stringify(clusters));
    return compact(clusters.map(cluster => {
        const status = get(cluster, 'status.status');
        if (status === 'ON') {
            const tieredCapacity = get(cluster, 'capacityFeatures.objectStoreConfigExtended.objectStoreConfig.usedSpace');
            const clusterUuid = get(cluster, 'ontapClusterProperties.clusterUuid');
            if (tieredCapacity) {
                const tieredCapacityGB = Math.ceil(capacityToBytes(tieredCapacity) / (1024 ** 3));
                logger.info('Charging Tiered capacity', tieredCapacity, 'in GB', tieredCapacityGB);
                return { resourceId: clusterUuid, val: tieredCapacityGB };
            }
            return { resourceId: clusterUuid, val: 0 };
        }
        return undefined;
    }));
}

// should return an object : subscription => {accounts: [{accountId, resources}], subscriptionId}
async function createChargingObjects(accountsToSubscriptions, timestamp) {
    return Object.keys(accountsToSubscriptions).reduce(async (accumulator, accountId) => {
        const resolvedAcc = await accumulator;
        try {
            const resources = await findTieringUsagePerAccount(accountId);
            if (resources.length) {
                const subscription = accountsToSubscriptions[accountId];
                const { subscriptionId } = subscription;
                if (resolvedAcc[subscriptionId]) {
                    resolvedAcc[subscriptionId].accounts.push({ accountId, resources });
                } else {
                    resolvedAcc[subscriptionId] = {
                        accounts: [{ accountId, resources }], subscriptionId, timestamp, dimension: CHARGING_DIMENSION
                    };
                }
            }
        } catch (err) {
            logger.error(`Error trying to create subscription to account ( with resources ) map function with account ${accountId} `, err);
        }
        return resolvedAcc;
    }, Promise.resolve({}));
}

async function collectChargingInfo(timestamp) {
    try {
        // manipulate subscriptions :
        // #1 find expired accounts
        // #2 list all subscriptions related to the service
        // #3 create an object account => subscriptions ( include subscriptionStatus )
        // intersect with expired accounts
        // #4 find relevant accounts ( expired ) in the map
        const candidateAccounts = await findAccountsWithoutFreeTrial();
        logger.info('Charging Candidate accounts for charging', JSON.stringify(candidateAccounts, null, 4));
        const cloudTieringSubscriptions = await listAllSubscriptionsForService();
        logger.info('Charging cloud-tiering subscriptions ', JSON.stringify(cloudTieringSubscriptions, null, 4));
        const accountsToSubscriptions = createAccountToSubscriptionMap(cloudTieringSubscriptions, candidateAccounts);
        // at this point account => [subscription]
        logger.info('Charging accountToChargeAndSubscription ', JSON.stringify(accountsToSubscriptions, null, 4));

        // create another object {subscriptionId => [{ accountId: account, resources }]}
        const subscriptions = await createChargingObjects(accountsToSubscriptions, timestamp);
        logger.info('Charging subscriptions to charge ', JSON.stringify(subscriptions, null, 4));

        const charging = Object.values(subscriptions);
        // dont send to charging service if there is no charging to charge - empty array
        if (charging.length) {
            chargingLogger.info('CHARGING is ', JSON.stringify(charging));
            await charge(charging);
        }
    } catch (err) {
        logger.error('CHARGING error', err);
    }
}

module.exports = {
    findAccountsWithoutFreeTrial,
    createAccountToSubscriptionMap,
    findTieringUsagePerAccount,
    createChargingObjects,
    collectChargingInfo
};
