const ms = require('ms');
const { ApolloError } = require('apollo-server');
const Promise = require('bluebird');
const {
    compact, round, get, pick, sumBy, head
} = require('lodash');
const {
    clusterInfo, configureFabricPoolResources, clusterMetadata, discoverCluster, enableColdDataReporting, configureAzureFabricPoolResources
} = require('../lib/cloud-manager/clusters');
const { getAssociatedSubscriptions, getAzureLocations } = require('../lib/cloud-manager/azure-metadata');
const createLicenseAndApply = require('../operations/ipa-operations');
const { getAllClustersInAccount } = require('../operations/multi-tenancy-operation');
const { getAggregates, attachObjectStore } = require('../lib/cloud-manager/aggregates');
const { getAgent } = require('../lib/cloud-manager/agents');
const {
    SAVING_OPPORTUNITY_ESTIMATION_RATIO, storageClasses, cloudProviders,
    AUTO, AWS, AZURE, occmProviderToCloudProviders
} = require('../utils/consts');
const HttpError = require('../utils/errors');
const { modifyVolume, getVolumes } = require('../lib/cloud-manager/volumes');
const { AWS_REGIONS } = require('../lib/aws/ec2');
const capacityToBytes = require('../utils/utils');
// const { createLicenseAndApply } = require('./ipa-operations');
const getLogger = require('../utils/logger/logger');

const INITIAL_LICENSE_CAPACITY_TB = 10;

const logger = getLogger();

const configureFabricPool = {
    [AWS]: {
        handler: async (clusterId, providerProperties) => {
            const tierLevel = providerProperties.storageClass ? storageClasses[AWS][providerProperties.storageClass] : 'ia';
            return configureFabricPoolResources(clusterId, providerProperties.region, providerProperties.awsKeys, tierLevel, providerProperties.ipSpace);
        }
    },
    [AZURE]: {
        handler: async (clusterId, providerProperties) => {
            // tier level is always hot transferring the result
            configureAzureFabricPoolResources(clusterId, providerProperties.region, providerProperties.ipSpace);
        }
    }
};


async function waitForFabricPoolToBeCreated(clusterId) {
    let interval;
    let timeout;

    const setTimeoutTime = process.env.IS_SIMULATOR ? '3s' : '30s';
    const setIntervalTime = process.env.IS_SIMULATOR ? '1m' : '10m';

    const p1 = new Promise((resolve, reject) => {
        timeout = setTimeout(() => {
            clearInterval(interval);
            reject(new Error('Fabric pool resources were not created after 10 minutes'));
        }, ms(setIntervalTime));
    });

    const p2 = new Promise((resolve, reject) => {
        interval = setInterval(async () => {
            try {
                const cluster = await clusterInfo(clusterId);
                logger.info('received cluster info when checking if fabricPool resources were created', JSON.stringify(cluster));
                const objectStoreName = get(cluster, 'capacityFeatures.objectStoreConfigExtended.objectStoreConfig.objectStoreName');
                if (objectStoreName) {
                    logger.info(`Fabric pool resources were created on cluster ${clusterId}, object store name ${objectStoreName}`);
                    clearInterval(interval);
                    clearTimeout(timeout);
                    resolve(objectStoreName);
                }
            } catch (err) {
                logger.error(`Received error when trying to check if fabric pool resources were created cluster ${clusterId}`, err);
                clearInterval(interval);
                clearTimeout(timeout);
                reject(err);
            }
        }, ms(setTimeoutTime));
    });

    return Promise.race([p1, p2]);
}

/*
calculates capacity and estimation
*/
function calculatePerformanceCapacity(usedPerformanceCapacity, inactiveCapacityFromOccm) {
    // #1 used = hot + inactive
    // #2 if (tiered and inactive ) => hot = used - inactive
    // #3 if (!tiered and inactive ) => hot = used - inactive, tiered = 0
    // #4 if (! tiered and !inactive ) => tiered = 0 , inactive = 0.7*used hot = used - inactive
    // #5 if (tiered and !inactive) => cold + hot = used - tiered,  cold = 0.7*(used-tiered) hot = used - inactive
    // if inactiveCapacityFromOccm is 0 or null we estimate otherwise we dont
    const inactiveCapacity = inactiveCapacityFromOccm || round((usedPerformanceCapacity) * 0.7);
    return {
        inactiveCapacity,
        usedPerformanceCapacity,
        activeCapacity: usedPerformanceCapacity - inactiveCapacity,
        isEstimatedInactiveCapacity: Boolean(!inactiveCapacityFromOccm)
    };
}

async function findTieringStats(accountId) {
    logger.info(`find tiering stats for account ${accountId}`);
    const clusters = await getAllClustersInAccount(accountId);
    const numberOfClusters = clusters.length;
    const numberOfActiveClusters = clusters.filter(cluster => cluster.status.status === 'ON').length;
    const tieredData = clusters.map(cluster => {
        const tieredCapacity = get(cluster, 'capacityFeatures.objectStoreConfigExtended.objectStoreConfig.usedSpace', 0);
        return { tieredCapacity: capacityToBytes(tieredCapacity) };
    });

    return { tieredCapacity: sumBy(tieredData, 'tieredCapacity'), numberOfActiveClusters, numberOfClusters };
}


function adjustVolumeResponse(volume, isEligibleForTiering, tieringRestrictionReason) {
    const {
        name, svmName, aggregateName, tieringPolicy, tierInactiveUserDataPercent
    } = volume;

    const usedPerformanceCapacity = capacityToBytes(get(volume, 'usedSize'));
    const inactiveCapacityFromOccm = capacityToBytes(get(volume, 'tierInactiveUserData'));
    const tieredCapacity = capacityToBytes(get(volume, 'capacityTierUsedSize'));
    const usedPerformanceAndTieredCapacity = usedPerformanceCapacity + tieredCapacity;
    const performance = calculatePerformanceCapacity(usedPerformanceCapacity, inactiveCapacityFromOccm);

    const firstDetails = {
        name,
        svmName,
        aggregateName,
        tieringPolicy: tieringPolicy || 'none',
        capacity: capacityToBytes(get(volume, 'size')),
        tieredCapacity,
        isEligibleForTiering,
        tieringRestrictionReason,
        usedPerformanceAndTieredCapacity
    };

    const newVolumeResponse = { ...firstDetails, ...performance };
    newVolumeResponse.inactiveCapacityPercent = (newVolumeResponse.isEstimatedInactiveCapacity) ? SAVING_OPPORTUNITY_ESTIMATION_RATIO * 100 : tierInactiveUserDataPercent;
    return newVolumeResponse;
}

function volumeAggregates(volume) {
    return volume.constituentsAggregates.length > 0 ? volume.constituentsAggregates : [volume.aggregateName];
}

function restrictedVolumes(volumes, allAggregates) {
    return compact(volumes.map(volume => {
        const requestedAggregates = allAggregates.filter(aggregate => volumeAggregates(volume).find(volAggrs => volAggrs === aggregate.name));
        const restrictedAggregate = requestedAggregates.find(aggregate => !aggregate.objectStoreName && aggregate.tieringRestriction.isTieringRestricted);
        if (restrictedAggregate) {
            const isHardRestricted = requestedAggregates.some(aggregate => !aggregate.objectStoreName && aggregate.tieringRestriction.isTieringRestricted && !aggregate.isObjectStoreAttachEligible);
            return {
                name: volume.name,
                svmName: volume.svmName,
                isHardRestricted,
                restrictionReason: restrictedAggregate.tieringRestriction.restrictionReason || 'unknown restriction reason'
            };
        }
        return undefined;
    }));
}

async function getSupportedVolumesNoReason(clusterId) {
    try {
        logger.info(`Finding eligible and soft restricted volumes - without attach-object-store for clusterId ${clusterId}`);
        const [volumes, aggregates] = await Promise.all([getVolumes(clusterId), getAggregates(clusterId)]);
        const restrictedVols = restrictedVolumes(volumes, aggregates);
        return compact(volumes.map(volume => {
            const restrictedVolume = restrictedVols.find(restrictedVol => restrictedVol.name === volume.name && restrictedVol.svmName === volume.svmName);
            return (restrictedVolume && restrictedVolume.isHardRestricted) ? undefined : volume;
        }));
    } catch (err) {
        logger.error('Failed to get supported volumes no reason with error', err);
        throw err;
    }
}

async function getTieredVolumesSummary(clusterId) {
    const supportedVolumes = await getSupportedVolumesNoReason(clusterId);
    return {
        eligibleVolumes: supportedVolumes.length,
        tieredVolumes: supportedVolumes.filter(volume => volume.tieringPolicy && volume.tieringPolicy !== 'none').length
    };
}

function findOntapVersion(ontapVersion) {
    const ontapVersionRegex = /(\d+\.\d*|\d+)/;
    const version = ontapVersionRegex.exec(ontapVersion)[1];
    return parseFloat(version);
}

async function getSupportedVolumes(clusterId) {
    try {
        logger.info('Finding supported (eligible + soft) volumes');
        const [volumes, aggregates] = await Promise.all([getVolumes(clusterId), getAggregates(clusterId)]);
        const allRestrictedVols = restrictedVolumes(volumes, aggregates);

        return compact(volumes.map(volume => {
            const { state } = volume;
            if (state !== 'online') { // can be volume with good aggregates butwith offline state
                return adjustVolumeResponse(volume, false, `Volume’s state is ${volume.state}. To enable tiering, a volume’s state must be 'online'`);
            }
            const restrictedVolume = allRestrictedVols.find(restrictedVol => restrictedVol.name === volume.name && restrictedVol.svmName === volume.svmName);
            if (restrictedVolume) {
                if (!restrictedVolume.isHardRestricted) {
                    return adjustVolumeResponse(volume, false, restrictedVolume.restrictionReason);
                }
                return undefined;
            }
            volume.isEligibleForTiering = true;
            return adjustVolumeResponse(volume, true);
        }));
    } catch (err) {
        logger.error('Failed to get supported volumes with error', err);
        throw err;
    }
}

async function validateTiering(clusterId, tieringPolicy, ipSpace) {
    const clusterInformation = await clusterInfo(clusterId);
    const { status: { status: clusterStatus }, ontapClusterProperties } = clusterInformation;
    if (clusterStatus === 'ON') {
        const { ontapVersion } = ontapClusterProperties;
        const parsedOntapVersion = findOntapVersion(ontapVersion);
        if (parsedOntapVersion < 9.4 && tieringPolicy === AUTO) {
            throw new ApolloError(`tieringPolicy ${AUTO} is allowed from ontap version 9.4 and above`, 400);
        }
        if (ipSpace) { // upSpace is mandtory on setupTiering - if it is here we are validaing setuptiering and not addToTier
            // validate the ipspace is in the cluster and it is on
            const { lifs } = ontapClusterProperties;
            const interClusterLifwithIpSpace = lifs.find(lif => lif.ipSpace === ipSpace && lif.role === 'intercluster');
            if (!interClusterLifwithIpSpace) {
                throw new ApolloError(`There is no intercluster lif that contains ipSpace ${ipSpace} on cluster ${clusterId}`, 403);
            }
        }
    } else {
        throw new ApolloError(`cluster with id ${clusterInformation.publicId} is not "on"`, 400);
    }
}

function volumesTostring(volumes) {
    return JSON.stringify(volumes.map(vol => `${vol.name}, ${vol.svmName}`));
}

async function attachObjectSoreAndModifyTieringPolicy(clusterId, volumeIdentifiers, objectStoreName, tieringPolicy) {
    logger.info('About to attach and modify volumes');
    const [volumes, aggregates] = await Promise.all([getVolumes(clusterId), getAggregates(clusterId)]);
    const requestedVolumes = volumes.filter(volume => volumeIdentifiers.find(volumeIdentifier => volumeIdentifier.name === volume.name && volumeIdentifier.svmName === volume.svmName));
    logger.info('Requested volumes:', volumesTostring(requestedVolumes));
    const restrictedVols = restrictedVolumes(requestedVolumes, aggregates);
    logger.info('Restricted volumes:', volumesTostring(restrictedVols));
    const eligibleVolumes = requestedVolumes.filter(requestedVol => !restrictedVols.find(restrictedVol => restrictedVol.name === requestedVol.name && restrictedVol.svmName === requestedVol.svmName) && requestedVol.state === 'online');
    logger.info('Eligible volumes:', volumesTostring(eligibleVolumes));
    const aggregatesToAttach = eligibleVolumes.map(vol => volumeAggregates(vol)).flat();
    const aggregatesToAttachUnique = [...new Set(aggregatesToAttach)];
    logger.info('Going to attach aggregates:', JSON.stringify(aggregatesToAttachUnique));

    // we attach object store to all eligable aggregates
    await Promise.all(aggregatesToAttachUnique.map(aggregateName => attachObjectStore(clusterId, aggregateName, false, objectStoreName)));

    // update tiering policy
    await Promise.map(eligibleVolumes, async eligibleVolume => {
        try {
            await modifyVolume(clusterId, eligibleVolume.svmName, eligibleVolume.name, tieringPolicy || AUTO);
        } catch (err) {
            logger.error(`Failed to modify volume ${eligibleVolume.name} from cluster ${clusterId} to tiering policy ${tieringPolicy || AUTO} ${err.message}. Skipping...`, err);
        }
    }, { concurrency: 20 });
}

async function addToTier(clusterId, volumeIdentifiers, tieringPolicy) {
    logger.info('Continue tiering already tiered cluster ... ', { clusterId, volumeIdentifiers, tieringPolicy });
    const clusterInformation = await clusterInfo(clusterId);
    logger.info('Cluster information:', JSON.stringify(clusterInformation));
    // get or create object store name
    const objectStoreName = get(clusterInformation, 'capacityFeatures.objectStoreConfigExtended.objectStoreConfig.objectStoreName');
    if (!objectStoreName) {
        throw new HttpError('No object store attached - you should use startTiering mutation', 403);
    }
    await attachObjectSoreAndModifyTieringPolicy(clusterId, volumeIdentifiers, objectStoreName, tieringPolicy);
}

async function setupTiering(cloudProvider, clusterId, region, awsKeys, volumeIdentifiers, storageClass, tieringPolicy, ipSpace) {
    logger.info('Setup tiering:', JSON.stringify({
        clusterId, region, volumeIdentifiers, storageClass, cloudProvider, tieringPolicy, ipSpace
    }));

    const clusterInformation = await clusterInfo(clusterId);

    try {
        await createLicenseAndApply(clusterId, INITIAL_LICENSE_CAPACITY_TB);
        logger.info(`license apply on ${clusterId} successfully executed`);
    } catch (err) {
        logger.error(`Failed to create and apply license on ${clusterInformation.name}. skipping...`, err);
    }

    logger.info('Cluster information inside setupTiering:', JSON.stringify(clusterInformation));
    // get or create object store name
    let objectStoreName = get(clusterInformation, 'capacityFeatures.objectStoreConfigExtended.objectStoreConfig.objectStoreName');
    if (!objectStoreName) {
        logger.info('Create objectStore and wait for results');
        await configureFabricPool[cloudProvider].handler(clusterId, {
            region, storageClass, awsKeys, ipSpace
        });
        (objectStoreName = await waitForFabricPoolToBeCreated(clusterId));
    }
    await attachObjectSoreAndModifyTieringPolicy(clusterId, volumeIdentifiers, objectStoreName, tieringPolicy);
}

function validateEligibleForTiering(ontapVersionParsed, clusterAddress, agentProvider) {
    logger.info(`Validating if cluster with address ${clusterAddress}, ONTAP version ${ontapVersionParsed} and cloud provider: ${agentProvider} is eligable for tiering`);
    const eligibleVersionPerProvider = agentProvider === cloudProviders.AZURE ? 9.4 : 9.2;
    if (ontapVersionParsed < eligibleVersionPerProvider) {
        throw new HttpError(`Can't discover ONTAP Cluster. ONTAP Version ${ontapVersionParsed} is not supported`, 403);
    }
}

// for savings - inactiveOrTieredCapacityGB is tieredCapacity for opportunities - inactiveCapacity
function savingsFunction(tieredandUsedCapacityBytes, inactiveOrTieredCapacityBytes) {
    const TIER_LICENSE_PER_GB = 0.033;
    const S3_ESTIMATED_COST_PER_GB = 0.0152;
    const FIVE_YEAR_AFF_TCO_PER_ALLOCATED_GB = 3.042;
    const UTILIZATION = 0.4;
    const fiveYearTierTcoPerGb = (S3_ESTIMATED_COST_PER_GB + TIER_LICENSE_PER_GB) * 60;
    const fivYearAffTcpPerRawGb = FIVE_YEAR_AFF_TCO_PER_ALLOCATED_GB / UTILIZATION;
    const fiveYearWithTierTcoPerGB = (fivYearAffTcpPerRawGb * (tieredandUsedCapacityBytes - inactiveOrTieredCapacityBytes) + fiveYearTierTcoPerGb * inactiveOrTieredCapacityBytes) / tieredandUsedCapacityBytes;
    const fiveYearsSavingsPerGB = round(fivYearAffTcpPerRawGb - fiveYearWithTierTcoPerGB, 2);
    return round(fiveYearsSavingsPerGB * tieredandUsedCapacityBytes / (1024 ** 3));
}

function calculateSavingsOpportunitiesAndSavings(performanceAndTieredCapacity, tieredCapacity, inactiveCapacityFromOccm) {
    logger.info(`Calculating savings opportunities and savings received ${JSON.stringify({
        performanceAndTieredCapacity, tieredCapacity, inactiveCapacityFromOccm
    })}`);
    // if inactiveCapacityFromOccm is 0 or null we estimate otherwise we dont
    const inactiveCapacity = inactiveCapacityFromOccm || round((performanceAndTieredCapacity - tieredCapacity) * 0.7);

    return {
        savings: savingsFunction(performanceAndTieredCapacity, tieredCapacity),
        savingOpportunities: savingsFunction(performanceAndTieredCapacity, inactiveCapacity)
    };
}

function regionByS3Endpoint(s3Endpoint) {
    let regionKey;
    if (s3Endpoint === 's3.amazonaws.com') {
        regionKey = 'us-east-1';
        return AWS_REGIONS[regionKey];
    }
    const awsRegionRegex = /s3-(\S*).amazonaws.com/;
    const regexResult = awsRegionRegex.exec(s3Endpoint);
    if (regexResult) {
        ([, regionKey] = regexResult);
        return AWS_REGIONS[regionKey];
    }

    // in case we talk about azure we need to return the endPoint for now
    return s3Endpoint;
}

async function getAzureRegions(agentId) {
    // #1 get subscription
    const associatedSubscriptions = await getAssociatedSubscriptions(agentId);

    // #2 find default subscription, if there is no default subscription pick the first one
    const subscription = associatedSubscriptions.find(associatedSubscription => associatedSubscription.default) || head(associatedSubscriptions);

    // #3 get regions by subscription
    const regions = await getAzureLocations(subscription.subscriptionId, agentId);

    return regions.map(region => ({ region: region.name, regionName: region.displayName }));
}

async function adjustClusterResponse(cluster) {
    const {
        publicId: clusterId, capacityFeatures, status: { status: clusterStatus }, name, ontapClusterProperties, agentCloudProvider, agentId
    } = cluster;
    logger.info(`adjusting cluster response for cluster ${clusterId} with status ${clusterStatus} from ${agentCloudProvider}`);
    const inactiveCapacityFromOccm = capacityToBytes(get(capacityFeatures, 'inactiveUserData'));
    const performanceTierUsedCapacity = capacityToBytes(get(capacityFeatures, 'performanceTierUsedCapacity'));// performance tier used size including hard restricted
    const tieredCapacity = capacityToBytes(get(capacityFeatures, 'objectStoreConfigExtended.objectStoreConfig.usedSpace'));
    const performanceAndTieredCapacity = performanceTierUsedCapacity + tieredCapacity;
    const { savings, savingOpportunities } = calculateSavingsOpportunitiesAndSavings(performanceAndTieredCapacity, tieredCapacity, inactiveCapacityFromOccm);
    const providerType = get(capacityFeatures, 'objectStoreConfigExtended.objectStoreConfig.providerType');
    return {
        clusterId,
        name,
        status: {
            status: clusterStatus,
            message: get(cluster, 'status.failureCauses.invalidOntapCredentials') ? 'Invalid ONTAP credentials' : cluster.status.message
        },
        ontapClusterProperties: clusterStatus === 'ON' ? pick(ontapClusterProperties, ['clusterAddress', 'ontapVersion', 'clusterUuid']) : undefined,
        capacityInformation: clusterStatus === 'ON' ? {
            usedPerformanceAndTieredCapacity: performanceAndTieredCapacity,
            performanceCapacity: calculatePerformanceCapacity(performanceTierUsedCapacity, inactiveCapacityFromOccm),
            physicalCapacity: capacityToBytes(get(capacityFeatures, 'rawCapacity')),
            savingOpportunities
        } : undefined,
        tierInformation: get(capacityFeatures, 'objectStoreConfigExtended.objectStoreConfig') ? {
            bucketName: get(capacityFeatures, 'objectStoreConfigExtended.objectStoreConfig.s3Name'),
            providerType: occmProviderToCloudProviders[providerType],
            tieredCapacity,
            objectStoreName: get(capacityFeatures, 'objectStoreConfigExtended.objectStoreConfig.objectStoreName'),
            regionName: providerType === 'aws_s3' ? regionByS3Endpoint(get(capacityFeatures, 'objectStoreConfigExtended.objectStoreConfig.server')) : undefined,
            tierLevel: storageClasses[occmProviderToCloudProviders[providerType]][get(capacityFeatures, 'objectStoreConfigExtended.tierLevel')],
            savings
        } : undefined,
        agentCloudProvider,
        agentId
    };
}

async function updateColdDataReporting(ontapVersionParsed, clusterId, clusterAddress) {
    if (ontapVersionParsed >= 9.4) {
        logger.info(`About to enable cold data reporting on cluster ${clusterAddress}`);
        enableColdDataReporting(clusterId);
    } else {
        logger.info(`Can't enable cold data reporting on cluster ${clusterAddress} due to its ONTAP version: ${ontapVersionParsed}. Skipping...`);
    }
}

async function discoverClusterOperation(clusterAddress, clusterUserName, clusterPassword, workspaceId, agentId, accountId) {
    const agent = await getAgent(agentId, accountId);
    const { agent: { status: agentStatus, name: agentName, provider: agentProvider } } = agent;
    logger.info('agentStatus inside discover', agentStatus);

    if (agentStatus === 'active') {
        const metadata = await clusterMetadata(clusterAddress, clusterUserName, clusterPassword, agentId);
        logger.info('clusterMetadata received', JSON.stringify(metadata));
        const { ontapVersion } = metadata;
        const ontapVersionParsed = findOntapVersion(ontapVersion);
        validateEligibleForTiering(ontapVersionParsed, clusterAddress, agentProvider);
        logger.info(`Cluster with address ${clusterAddress} is eligable for tiering - trying to discover with agent ${agentId} and workspaceId ${workspaceId}`);
        const clusterId = await discoverCluster(metadata.clusterName, clusterAddress, clusterUserName, clusterPassword, workspaceId, agentId);
        logger.info(`cluster ${clusterId} was discovered in account ${accountId} and workspace ${workspaceId} with agentId ${agentId}`);
        updateColdDataReporting(ontapVersionParsed, clusterId, clusterAddress);
        return clusterId;
    }
    logger.error(`Service Connector with name ${agentName} and id ${agentId} is not connected- it has status ${agentStatus}`);
    throw new ApolloError('"Service Connector" is not active', 403);
}

async function getIpSpaces(clusterId) {
    const clusterInformation = await clusterInfo(clusterId);
    const { ontapClusterProperties, status: { status } } = clusterInformation;
    if (status === 'ON') {
        const { lifs } = ontapClusterProperties;
        return [...new Set(lifs.filter(lif => {
            const { role, ipSpace } = lif;
            return ipSpace && role === 'intercluster';
        }).map(lif => lif.ipSpace))];
    }
    throw new ApolloError(`Cluster with id ${clusterId} has status ${status}.`, 403);
}

module.exports = {
    setupTiering,
    addToTier,
    calculateSavingsOpportunitiesAndSavings,
    adjustClusterResponse,
    calculatePerformanceCapacity,
    getSupportedVolumes,
    getSupportedVolumesNoReason,
    getTieredVolumesSummary,
    discoverClusterOperation,
    validateEligibleForTiering,
    findTieringStats,
    regionByS3Endpoint,
    getAzureRegions,
    validateTiering,
    updateColdDataReporting,
    getIpSpaces
};
