const { compact } = require('lodash');
const config = require('config');
const Account = require('../models/account');
const getLogger = require('../utils/logger/logger');
const { isLicensedAccountInternal } = require('../operations/subscription-operations');
const { getAllClustersInAccountInternal } = require('../operations/multi-tenancy-operation');
const { licenseStatus } = require('../lib/cloud-manager/clusters');
const createLicenseAndApply = require('../operations/ipa-operations');
const capacityToBytes = require('../utils/utils');


const logger = getLogger();

async function updateLicense(accountId) {
    const licensedAccount = await isLicensedAccountInternal(accountId);
    logger.info(`Checking if need to update capacity licenses in account: ${accountId}`);
    if (licensedAccount) {
        const clusters = await getAllClustersInAccountInternal(accountId);
        const promises = clusters.map(async onprem => {
            try {
                const { licenseInformation } = await licenseStatus(onprem.publicId);
                const licenseCapacityRatio = capacityToBytes(licenseInformation.currentCapacity) / capacityToBytes(licenseInformation.maxCapacity);
                const increaseLicenseCapacityRatio = config.get('increase_license_capacity_ratio');
                logger.info(`License capacity ratio of cluster ${onprem.publicId} in ${accountId} is: ${licenseCapacityRatio}, license capcity threshold: ${increaseLicenseCapacityRatio}`);
                if (capacityToBytes(licenseInformation.currentCapacity) / capacityToBytes(licenseInformation.maxCapacity) >= increaseLicenseCapacityRatio) {
                    const newLicenseCapacity = licenseInformation.maxCapacity.size + 10;
                    logger.info(`Going to update license of accountId: ${accountId}, clusterId: ${onprem.publicId} with license capacity of ${newLicenseCapacity}`);
                    return await createLicenseAndApply(onprem.publicId, newLicenseCapacity);
                }
            } catch (err) {
                logger.error(`Failed to update license for Cluster ${onprem.publicId}. Skipping...`, err);
            }
        });
        return compact(Promise.all(promises));
    }
    logger.info(`Account: ${accountId} is not a licensed account`);
}

async function updateLicenses() {
    const accounts = await Account.find();
    logger.info('Starting update license check cycle');
    accounts.forEach(async account => {
        try {
            await updateLicense(account.accountId);
            logger.info('Successfully finished update license check cycle');
        } catch (err) {
            logger.error(`Failed to check/update license for account ${account.name} with ID: ${account.accountId}. Skipping...`, err);
        }
    });
}

module.exports = { updateLicenses, updateLicense };
