const {
    isEmpty, first, sortBy, compact
} = require('lodash');
const moment = require('moment');
const {
    getAccounts, createAccount, getWorkspaces, createWorkspace, getResourcesByWorkspace, getResourcesByAccount, getResourcesByAccountInternal
} = require('../lib/cloud-manager/multi-tenancy');
const { getOccmServiceToken, jwtGetUserInfo } = require('../lib/auth0');
const { getAgent } = require('../lib/cloud-manager/agents');
const { clusterInfo } = require('../lib/cloud-manager/clusters');
const { CLOUD_TIERING_NAMESPACE, FREE_TRIAL_DAYS } = require('../utils/consts');
const HttpError = require('../utils/errors');
const Account = require('../models/account');
const getLogger = require('../utils/logger/logger');

const logger = getLogger();


async function shouldAutomaticExtend() {
    const userInfo = await jwtGetUserInfo(CLOUD_TIERING_NAMESPACE.get('userToken'));
    const userMetadata = userInfo.data['http://cloud.netapp.com/user_metadata'];
    const email = userMetadata.email.trim().toLowerCase();
    return email.includes('@netapp');
}


async function findAccount() {
    const accounts = await getAccounts();
    logger.info(`received accounts ${JSON.stringify(accounts)}`);
    let account = first(sortBy(accounts, accountToChoose => accountToChoose.accountName));
    let accountToCreate;
    if (isEmpty(accounts)) {
        const accountName = 'Default';
        account = await createAccount(accountName);
        logger.info(`Account ${JSON.stringify(account)} was created`);
        const automaticExtension = await shouldAutomaticExtend();
        accountToCreate = { accountId: account.accountPublicId, automaticExtension };
        await Account.create(accountToCreate);
    } else {
        const accountFromDb = await Account.findOne({ accountId: account.accountPublicId }).exec();
        if (!accountFromDb) {
            const automaticExtension = await shouldAutomaticExtend();
            accountToCreate = { accountId: account.accountPublicId, automaticExtension };
            if (process.env.IS_SIMULATOR) {
                accountToCreate.freeTrialExpiryDate = account.freeTrialExpiryDate;
                if (automaticExtension) {
                    accountToCreate.freeTrialExpiryDate = moment().add(FREE_TRIAL_DAYS, 'days').valueOf();
                }
            }
            await Account.create(accountToCreate);
        } else if ((accountFromDb.freeTrialExpiryDate && moment(accountFromDb.freeTrialExpiryDate).diff(moment().startOf('day'), 'days') < 30 && accountFromDb.automaticExtension)) {
            accountFromDb.freeTrialExpiryDate = moment().add(FREE_TRIAL_DAYS, 'days').valueOf();
            await accountFromDb.save();
        }
    }
    return account.accountPublicId;
}

async function findWorkspace(accountPublicId) {
    const workspaces = await getWorkspaces(accountPublicId);
    logger.info(`received workspaces ${JSON.stringify(workspaces)} of account ${accountPublicId}`);
    let workspace = first(sortBy(workspaces, workspaceToChoose => workspaceToChoose.workspaceName));
    if (isEmpty(workspaces)) {
        const workspaceName = 'Default';
        workspace = await createWorkspace(accountPublicId, workspaceName);
    }
    return workspace.workspacePublicId;
}


async function getClustersByResourcesInternal(resources, accountId, chosenClusterId) {
    const clusterIds = resources.map(resource => ({ clusterId: resource.resourceIdentifier, agentId: resource.agentId }));
    const filteredClusterIds = chosenClusterId ? clusterIds.filter(cluster => cluster.clusterId === chosenClusterId) : clusterIds;
    const promises = filteredClusterIds.map(async cluster => {
        const { clusterId } = cluster;
        try {
            logger.info(`Trying to receive internal cluster information for ${clusterId}`);
            return clusterInfo(clusterId, getOccmServiceToken(), accountId);
        } catch (err) {
            logger.error(`Failed receiving cluster information for cluster ${clusterId}`);
        }
    });
    return Promise.all(promises);
}

function getClustersByResources(resources, accountId, chosenClusterId) {
    const clusterIds = resources.map(resource => ({ clusterId: resource.resourceIdentifier, agentId: resource.agentId }));
    const filteredClusterIds = chosenClusterId ? clusterIds.filter(cluster => cluster.clusterId === chosenClusterId) : clusterIds;
    const promises = filteredClusterIds.map(async cluster => {
        const { agentId, clusterId } = cluster;
        try {
            logger.info(`Trying to receive cluster information for ${clusterId}`);
            const clusterDetails = await clusterInfo(clusterId);
            const agent = await getAgent(agentId, accountId);
            clusterDetails.agentCloudProvider = agent.agent.provider.toUpperCase();
            return clusterDetails;
        } catch (err) {
            logger.error(`Failed receiving cluster information for cluster ${clusterId}`);
        }
    });
    return Promise.all(promises);
}

async function getAgentIdByClusterId(clusterId, accountId) {
    // get all resources
    const resources = await getResourcesByAccount(accountId);
    // filter by clusterId
    const foundResource = resources.find(resource => resource.resourceIdentifier === clusterId);
    if (foundResource) {
        const { agentId } = foundResource;
        return agentId;
    }
    throw new HttpError(`Cluster with id ${clusterId} was not found`, 404);
}

async function getAllClustersInWorkspace(accountId, workspaceId, clusterId) {
    const resources = await getResourcesByWorkspace(accountId, workspaceId);
    logger.info(`Received resources of accountId ${accountId} and workspaceId ${workspaceId} : ${JSON.stringify(resources)}`);
    const clusters = await getClustersByResources(resources, accountId, clusterId);
    return compact(clusters);
}

async function getAllClustersInAccount(accountId, clusterId) {
    const resources = await getResourcesByAccount(accountId);
    logger.info(`Received resources of accountId ${accountId} : ${JSON.stringify(resources)}`);
    const clusters = await getClustersByResources(resources, accountId, clusterId);
    return compact(clusters);
}

async function getAllClustersInAccountInternal(accountId, clusterId) {
    const resources = await getResourcesByAccountInternal(accountId);
    logger.info(`Internal clusters Received resources of accountId ${accountId} : ${JSON.stringify(resources)}`);
    const clusters = await getClustersByResourcesInternal(resources, accountId, clusterId);
    logger.info('Internal clusters after get custer info', JSON.stringify(clusters));
    return compact(clusters);
}

module.exports = {
    findWorkspace,
    findAccount,
    getAllClustersInWorkspace,
    getAllClustersInAccount,
    getAllClustersInAccountInternal,
    getAgentIdByClusterId
};
