const { sortBy } = require('lodash');
const moment = require('moment');
const Account = require('../models/account');
const { listSubscriptionsOfAccount, listAllSubscriptionsForService } = require('../lib/subscription-service');
const {
    SUBSCRIBED, CLOUD_TIERING_NAMESPACE,
    UNSUBSCRIBED, UNSUBSCRIBE_PENDING, PENDING_SUBSCRIPTION, TIERING_SERVICE_NAME_SUBSCRIPTION
} = require('../utils/consts');

const getLogger = require('../utils/logger/logger');

const logger = getLogger();

async function calculateDaysLeftForTrial(accountId) {
    const account = await Account.findOne({ accountId });
    const { freeTrialExpiryDate } = account;
    if (freeTrialExpiryDate) {
        const daysLeftForTrial = moment(freeTrialExpiryDate).startOf('day').diff(moment().startOf('day'), 'days');
        return (daysLeftForTrial >= 0) ? daysLeftForTrial : -1;
    }
}

function findSubscription(subscriptionList) {
    logger.info('Find subscription to charge', JSON.stringify(subscriptionList));
    const sortedSubscriptions = sortBy(subscriptionList, 'createdAt');
    const subscribedSubscriptions = sortedSubscriptions.find(subscription => subscription.subscriptionStatus === SUBSCRIBED);
    if (subscribedSubscriptions) {
        return subscribedSubscriptions;
    }
    return sortedSubscriptions[0];
}

function subscriptionStatusBase(subscriptionsList) {
    const serviceSubscriptionList = subscriptionsList.filter(subscription => subscription.serviceName === TIERING_SERVICE_NAME_SUBSCRIPTION);
    if (serviceSubscriptionList.length) {
        const subscription = findSubscription(serviceSubscriptionList);
        return [UNSUBSCRIBE_PENDING, UNSUBSCRIBED, PENDING_SUBSCRIPTION].includes(subscription.subscriptionStatus) ? UNSUBSCRIBED : subscription.subscriptionStatus;
    }
    return UNSUBSCRIBED;
}

async function findSubscriptionStatus(accountId) {
    const subscriptionsList = await listSubscriptionsOfAccount(accountId);
    return subscriptionStatusBase(subscriptionsList);
}


async function findSubscriptionStatusInternal(accountId) {
    const serviceSubscriptions = await listAllSubscriptionsForService();
    const subscriptionsList = serviceSubscriptions.filter(subscription => subscription.accounts.includes(accountId));
    return subscriptionStatusBase(subscriptionsList);
}

async function findAccountInfo() {
    const accountId = CLOUD_TIERING_NAMESPACE.get('accountId');
    const daysLeftForTrial = await calculateDaysLeftForTrial(accountId);
    const subscriptionStatus = await findSubscriptionStatus(accountId);
    return { accountId, daysLeftForTrial, subscriptionStatus };
}

async function findAccountInfoInternal(accountId) {
    const daysLeftForTrial = await calculateDaysLeftForTrial(accountId);
    const subscriptionStatus = await findSubscriptionStatusInternal(accountId);
    return { accountId, daysLeftForTrial, subscriptionStatus };
}

async function isLicensedAccountInternal(accountId) {
    const { daysLeftForTrial, subscriptionStatus } = await findAccountInfoInternal(accountId);
    return (daysLeftForTrial >= 0 || subscriptionStatus === SUBSCRIBED);
}

module.exports = {
    calculateDaysLeftForTrial,
    findSubscription,
    findSubscriptionStatus,
    findAccountInfo,
    isLicensedAccountInternal
};
