const jwt = require('jsonwebtoken');
const jwksClient = require('jwks-rsa');
const { trimStart } = require('lodash');
const { JWT_TOKEN_AUTH0 } = require('./consts');
const getLogger = require('./logger/logger');

const logger = getLogger();

function getSigningKey(baseUrl, jwtToken) {
    return new Promise((resolve, reject) => {
        const token = jwt.decode(jwtToken, { complete: true });

        if (!token) { reject(new Error('Invalid JWT token')); }

        const client = jwksClient({
            cache: true,
            cacheMaxEntries: 60,
            rateLimit: true,
            jwksRequestsPerMinute: 30,
            jwksUri: `${baseUrl}/.well-known/jwks.json`
        });

        client.getSigningKey(token.header.kid, (err, key) => {
            if (err) {
                reject(err);
            } else {
                resolve(key.publicKey);
            }
        });
    });
}

function verify(jwtToken, signingKey, audience, baseUrl) {
    return new Promise((resolve, reject) => {
        const opts = {
            audience,
            issuer: `${baseUrl}/`,
            algorithms: ['RS256']
        };

        jwt.verify(jwtToken, signingKey, opts, (error, payload) => {
            if (error) {
                reject(error);
            } else {
                logger.debug('Successfully verified JWT token. Payload:', payload);
                resolve(payload);
            }
        });
    });
}

async function jwtVerify(jwtToken, audience) {
    const jwtTokenWithNoBearer = trimStart(jwtToken, 'Bearer').trim();

    try {
        if (process.env.IS_SIMULATOR && process.env.NODE_ENV === 'development') {
            return jwt.decode(jwtTokenWithNoBearer);
        }
        const signingKey = await getSigningKey(JWT_TOKEN_AUTH0, jwtTokenWithNoBearer);
        return verify(jwtTokenWithNoBearer, signingKey, audience, JWT_TOKEN_AUTH0);
    } catch (err) {
        logger.error('Error while verifying jwt token', err);
        throw err;
    }
}

module.exports = jwtVerify;
