const log4js = require('log4js');
const config = require('config');
const stackTrace = require('stack-trace');
const {
    cloneDeep, isPlainObject, isString, isArray, isError
} = require('lodash');
const path = require('path');
const { readFileSync } = require('fs');
const { format } = require('util');

const { CLOUD_TIERING_NAMESPACE } = require('../consts');

function hideSecretsHelper(object) {
    if (isArray(object)) {
        object.forEach(arrayObj => hideSecretsHelper(arrayObj));
    } else if (isPlainObject(object)) {
        Object.keys(object).forEach(key => {
            if (isPlainObject(object[key])) {
                hideSecretsHelper(object[key]);
            } else if (['access_token', 'client_secret', 'accessKey', 'secretKey'].includes(key)) {
                object[key] = '******';
            }
        });
    }
}

function hideSecrets(logEventData) {
    try {
        const jsonObj = JSON.parse(logEventData);
        hideSecretsHelper(jsonObj);
        return JSON.stringify(jsonObj);
    } catch (err) {
        if (!isString(logEventData)) {
            hideSecretsHelper(logEventData);
        }
        return logEventData;
    }
}

function configureLayoutWithToken() {
    const configFileContent = JSON.parse(readFileSync(config.get('paths.log4js-config-file')));
    Object.keys(configFileContent.appenders).forEach(appenderType => {
        const { layout } = configFileContent.appenders[appenderType];
        if (layout) {
            const { pattern } = layout;
            if (pattern) {
                layout.tokens = {
                    requestId: () => CLOUD_TIERING_NAMESPACE.get('requestId') || 'system',
                    accountId: () => CLOUD_TIERING_NAMESPACE.get('accountId') || 'system',
                    userName: () => CLOUD_TIERING_NAMESPACE.get('userName') || CLOUD_TIERING_NAMESPACE.get('task') || 'unknown',
                    message: logEvent => format(...logEvent.data.map(obj => {
                        if (isError(obj)) {
                            return hideSecrets(obj);
                        }
                        // used to return hideSecrets(obj) without cloneDeep.
                        const objClone = cloneDeep(obj);
                        return hideSecrets(objClone);
                    }))
                };
            }
        }
    });

    log4js.configure(configFileContent);
}

configureLayoutWithToken();


function getLogger(category = 'server') {
    const logger = log4js.getLogger(category);
    const trace = stackTrace.parse(new Error('something went wrong'));
    logger.addContext('module', trace[1].getFileName().replace(/(^.*[\\/]src[\\/])|(^.*[\\/]cloudtiering[\\/])/, '').split(path.sep).join('.'));
    return logger;
}

module.exports = getLogger;
