const { pick } = require('lodash');
const moment = require('moment');
const mongoose = require('mongoose');
const Account = require('../../src/models/account');
const {
    DB_CONNECTION_STRING, UNSUBSCRIBED, CHARGING_DIMENSION, UNSUBSCRIBE_PENDING, SUBSCRIBED
} = require('../../src/utils/consts');
const {
    findAccountsWithoutFreeTrial, createAccountToSubscriptionMap,
    findTieringUsagePerAccount, createChargingObjects
} = require('../../src/operations/charging-operations');
const getLogger = require('../../src/utils/logger/logger');
const multiTenancyOperations = require('../../src/operations/multi-tenancy-operation');


const logger = getLogger();

jest.mock('../../src/operations/multi-tenancy-operation');

const status = SUBSCRIBED;
function createSubscription(subscriptionId, subscriptionStatus, createdAt, accounts) {
    return {
        subscriptionId,
        subscriptionStatus,
        createdAt,
        accounts
    };
}

function createAccount(name, accountId) {
    return { name, accountId, freeTrialExpiryDate: (new Date()).getTime() };
}

const clusterResponseNoTiering = {
    name: 'cluster2',
    publicId: 'cluster2',
    ontapClusterProperties: {
        ontapVersion: 'v1',
        clusterAddress: '2.2.2.2',
        clusterUuid: 'cluster2',
        nodeModels: ['node'],
        lifs: [{
            name: 'lif1',
            status: 'up',
            role: 'intercluster',
            ipSpace: 'ipSpace3'
        }]
    },
    status: { status: 'ON' },
    capacityFeatures: {
        objectStoreConfigExtended: null
    }
};
const clusterResponse = {
    name: 'cluster1',
    publicId: 'cluster1',
    ontapClusterProperties: {
        clusterUuid: 'clusterUuid1',
        ontapVersion: 'v1',
        clusterAddress: '1.1.1.1',
        nodeModels: ['node'],
        lifs: [{
            name: 'lif1',
            status: 'up',
            role: 'intercluster',
            ipSpace: 'ipSpace3'
        }]
    },
    status: {
        status: 'ON'
    },
    capacityFeatures: {
        objectStoreConfigExtended: {
            objectStoreConfig: {
                objectStoreName: 'name',
                usedSpace: {
                    size: 5,
                    unit: 'MB'
                },
                providerType: 'aws_s3',
                server: 's3.amazonaws.com'
            },
            tierLevel: 'normal'
        }
    }
};
const clusterResponse2 = {
    name: 'cluster2',
    publicId: 'cluster2',
    ontapClusterProperties: {
        ontapVersion: 'v1',
        clusterAddress: '2.2.2.2',
        clusterUuid: 'cluster2',
        nodeModels: ['node'],
        lifs: [{
            name: 'lif1',
            status: 'up',
            role: 'intercluster',
            ipSpace: 'ipSpace3'
        }]
    },
    status: { status: 'ON' },
    capacityFeatures: {
        objectStoreConfigExtended: {
            objectStoreConfig: {
                objectStoreName: 'name',
                usedSpace: {
                    size: 5,
                    unit: 'MB'
                },
                providerType: 'aws_s3',
                server: 's3.amazonaws.com'

            },
            tierLevel: 'normal'
        }
    }
};
const downClusterResponse = {
    name: 'AFF-FAILED-1',
    publicId: 'eda35168-d036-4cce-9cc8-ca916ec36329',
    ontapClusterProperties: {
        ontapVersion: '',
        clusterAddress: '11.11.11.11',
        clusterUuid: '10bec0b2-7c82-404f-868a-424cf5158b81'
    },
    capacityFeatures: null,
    status: {
        status: 'FAILED',
        failureCauses: {
            invalidOntapCredentials: true
        }
    }
};
const unknownClusterResponse = {
    publicId: 'clusterId',
    status: {
        status: 'UNKNOWN',
        message: 'agent is down'
    }
};


describe('findAccountsWithoutFreeTrial', () => {
    beforeAll(async () => {
        try {
            await mongoose.connect(DB_CONNECTION_STRING, { useNewUrlParser: true, dbName: 'cloudTiering' });
        } catch (e) {
            logger.error('Cannot connect to mongoDB');
        }
    });

    afterAll(async done => {
        try {
            await mongoose.connection.close();
            done();
        } catch (e) {
            logger.error('Error closing connection', e);
        }
    });

    beforeEach(() => Account.deleteMany({}));

    test('findAccountsWithoutFreeTrial- no account with expired trial ', async () => {
        const account = { name: 'test', accountId: '1' };
        await Account.create(account);
        const result = await findAccountsWithoutFreeTrial();
        expect(result).toHaveLength(0);
    });


    test('findAccountsWithoutFreeTrial- account to charge', async () => {
        const account = { name: 'test', accountId: '1', freeTrialExpiryDate: moment().subtract(3, 'days') };
        await Account.create(account);
        const result = await findAccountsWithoutFreeTrial();
        expect(result).toHaveLength(1);
    });

    test('findAccountsWithoutFreeTrial- no account to charge - in DB only account with trial that hasnt started yet', async () => {
        const account = { name: 'test', accountId: '1' };
        await Account.create(account);
        const result = await findAccountsWithoutFreeTrial();
        expect(result).toHaveLength(0);
    });
});

describe('createAccountToSubscriptionMap', () => {
    it('received an empty array of subscriptions and an array of candidate accounts, should return an empty object', () => {
        expect(createAccountToSubscriptionMap([], [createAccount('a1', '1')])).toEqual({});
    });

    it('received an empty array of subscriptions and an  empty array of candidate accounts, should return an empty object', () => {
        expect(createAccountToSubscriptionMap([], [])).toEqual({});
    });

    it('received an  array of subscriptions and an empty array of candidate accounts, should return an empty object', () => {
        expect(createAccountToSubscriptionMap(
            [createSubscription('subscription1', status, 1556636730921, ['a1'])], []
        )).toEqual({});
    });

    test('candidateAccount does not includes accounts in cloudTieringSubscriptions, should return an empty object', () => {
        expect(createAccountToSubscriptionMap(
            [
                createSubscription('subscription1', status, 1556636730921, ['a1'])], [createAccount('a2', 'a2')]
        )).toEqual({});
    });

    test('candidateAccount includes all accounts in cloudTieringSubscriptions, should return an object with all of the accounts', () => {
        const subscription1 = createSubscription('s1', status, (new Date()).getTime(), ['b1']);
        const subscription2 = createSubscription('s1', status, (new Date()).getTime(), ['c1']);
        const subscriptions = [subscription1, subscription2];
        const candidateAccounts = [createAccount('b1', 'b1'), createAccount('c1', 'c1')];
        expect(createAccountToSubscriptionMap(subscriptions, candidateAccounts)).toEqual({ b1: pick(subscription1, ['subscriptionId', 'subscriptionStatus', 'createdAt']), c1: pick(subscription2, ['subscriptionId', 'subscriptionStatus', 'createdAt']) });
    });

    test('there is one common account in candidateAccount  and accountToSubscriptions, should return an object with only this account', () => {
        const subscription1 = createSubscription('s1', status, (new Date()).getTime(), ['b1']);
        const subscription2 = createSubscription('s1', status, (new Date()).getTime(), ['d1']);
        const subscriptions = [subscription1, subscription2];
        const candidateAccounts = [createAccount('b1', 'b1')];
        expect(createAccountToSubscriptionMap(subscriptions, candidateAccounts)).toEqual({ b1: pick(subscription1, ['subscriptionId', 'subscriptionStatus', 'createdAt']) });
    });

    test('There are 2 subscriptions with 2 accounts, one in candidateAccounts and one not, should return an object with the account in candidates accounts', () => {
        const subscription1 = createSubscription('s1', status, (new Date()).getTime(), ['d1', 'b1']);
        const subscription2 = createSubscription('s1', status, (new Date()).getTime(), ['d1', 'b1']);
        const subscriptions = [subscription1, subscription2];
        const candidateAccounts = [createAccount('b1', 'b1')];
        expect(createAccountToSubscriptionMap(subscriptions, candidateAccounts)).toEqual({ b1: pick(subscription1, ['subscriptionId', 'subscriptionStatus', 'createdAt']) });
    });

    it('received a subscription array - each has one account- distinct accounts', () => {
        expect(createAccountToSubscriptionMap([createSubscription('subscription1', status, 1556636730921, ['a1']),
            createSubscription('subscription2', status, 1556636730921, ['a2'])], [createAccount('a2', 'a2'), createAccount('a1', 'a1')]))
            .toEqual({ a1: { subscriptionId: 'subscription1', subscriptionStatus: status, createdAt: 1556636730921 }, a2: { subscriptionId: 'subscription2', subscriptionStatus: status, createdAt: 1556636730921 } });
    });

    it('received a subscription array - 2 subscriptions with the same account , should return an object with one account and one subscription - second is older  ', () => {
        expect(createAccountToSubscriptionMap([createSubscription('subscription1', status, 1556636730921, ['a1']),
            createSubscription('subscription2', status, 1556636730920, ['a1'])], [createAccount('a2', 'a2'), createAccount('a1', 'a1')]))
            .toEqual({ a1: { subscriptionId: 'subscription2', subscriptionStatus: status, createdAt: 1556636730920 } });
    });

    it('received a subscription array - 2 subscriptions with the same account , should return an object with one account and one subscription  - first is older ', () => {
        expect(createAccountToSubscriptionMap([createSubscription('subscription1', status, 1556636730920, ['a1']),
            createSubscription('subscription2', status, 1556636730921, ['a1'])], [createAccount('a2', 'a2'), createAccount('a1', 'a1')]))
            .toEqual({ a1: { subscriptionId: 'subscription1', subscriptionStatus: status, createdAt: 1556636730920 } });
    });

    it('received a subscription array - 2 unsubscribed subscriptions with the same account , should return an  empty object', () => {
        expect(createAccountToSubscriptionMap([createSubscription('subscription1', UNSUBSCRIBED, 1556636730920, ['a1']),
            createSubscription('subscription2', UNSUBSCRIBED, 1556636730921, ['a1']), [createAccount('a2', 'a2'), createAccount('a1', 'a1')]]))
            .toEqual({});
    });

    it('received a subscription array - 2 subscriptions  one SUBSCRIBED and one UNSUBSCRIBED with the same account , should return an object with one account and one subscription  - first is older ', () => {
        expect(createAccountToSubscriptionMap([createSubscription('subscription1', status, 1556636730920, ['a1']),
            createSubscription('subscription2', UNSUBSCRIBED, 1556636730921, ['a1'])], [createAccount('a2', 'a2'), createAccount('a1', 'a1')]))
            .toEqual({ a1: { subscriptionId: 'subscription1', subscriptionStatus: status, createdAt: 1556636730920 } });
    });

    it('received a subscription array - 2 subscriptions  one SUBSCRIBED and one unsubscribe-pending , unsubscribed-pending is earlier, same account , should return an object with one account and one subscription  - first is older ', () => {
        expect(createAccountToSubscriptionMap([createSubscription('subscription1', UNSUBSCRIBE_PENDING, 1556636730920, ['a1']),
            createSubscription('subscription2', SUBSCRIBED, 1556636730921, ['a1'])], [createAccount('a2', 'a2'), createAccount('a1', 'a1')]))
            .toEqual({ a1: { subscriptionId: 'subscription2', subscriptionStatus: status, createdAt: 1556636730921 } });
    });

    it('received a subscription array - 2 subscriptions  one unsubscribed-pending and one unsubscribe-pending , unsubscribed-pending is earlier, same account , should return an object with one account and one subscription  - first is older ', () => {
        expect(createAccountToSubscriptionMap([createSubscription('subscription1', UNSUBSCRIBE_PENDING, 1556636730920, ['a1']),
            createSubscription('subscription2', UNSUBSCRIBE_PENDING, 1556636730919, ['a1'])], [createAccount('a2', 'a2'), createAccount('a1', 'a1')]))
            .toEqual({ a1: { subscriptionId: 'subscription2', subscriptionStatus: UNSUBSCRIBE_PENDING, createdAt: 1556636730919 } });
    });
});

describe('findTieringUsagePerAccount', () => {
    beforeEach(() => {
        multiTenancyOperations.getAllClustersInAccountInternal.mockClear();
    });

    test('findTieringUsagePerAccount- Happy path - 2 clusters in account both have tiering', async () => {
        multiTenancyOperations.getAllClustersInAccountInternal.mockReturnValue([clusterResponse, clusterResponse2]);
        const result = await findTieringUsagePerAccount('accountId');
        expect(result).toEqual([
            { resourceId: 'clusterUuid1', val: 1 },
            { resourceId: 'cluster2', val: 1 }]);
    });


    test('findTieringUsagePerAccount- no clusters in account', async () => {
        multiTenancyOperations.getAllClustersInAccountInternal.mockReturnValue([]);
        const result = await findTieringUsagePerAccount('accountId');
        expect(result)
            .toEqual([]);
    });

    test('findTieringUsagePerAccount- account has one cluster - no tiering', async () => {
        multiTenancyOperations.getAllClustersInAccountInternal.mockReturnValue([clusterResponseNoTiering]);
        const result = await findTieringUsagePerAccount('accountId1');
        expect(result)
            .toEqual([{ resourceId: 'cluster2', val: 0 }]);
    });


    test('findTieringUsagePerAccount- account has one cluster status is OFF - no tiering should return an empty array', async () => {
        multiTenancyOperations.getAllClustersInAccountInternal.mockReturnValue([downClusterResponse]);
        const result = await findTieringUsagePerAccount('accountId1');
        expect(result)
            .toEqual([]);
    });

    test('findTieringUsagePerAccount- account has one cluster status is UNKNOWN - no tiering should return an empty array', async () => {
        multiTenancyOperations.getAllClustersInAccountInternal.mockReturnValue([unknownClusterResponse]);
        const result = await findTieringUsagePerAccount('accountId1');
        expect(result)
            .toEqual([]);
    });
});

describe('createChargingObjects', () => {
    test('accountsToSubscriptions is an empty object should return an empty object', async () => {
        const timestamp = new Date().toISOString();
        expect(await createChargingObjects({}, timestamp)).toEqual({});
    });

    test('accountsToSubscriptions has one account with subscription but with no resources should return empty object', async () => {
        multiTenancyOperations.getAllClustersInAccountInternal.mockReturnValue([]);
        const timestamp = new Date().toISOString();
        expect(await createChargingObjects(
            { a1: createSubscription('subscriptionId', status, (new Date()).getTime(), ['a1']) }, timestamp
        )).toEqual({});
    });
    test('accountsToSubscriptions has one account and subscription with resources hat are not tiered should return an object that includes the account and the subscription', async () => {
        multiTenancyOperations.getAllClustersInAccountInternal.mockReturnValue([clusterResponseNoTiering]);
        const timestamp = new Date().toISOString();
        expect(await createChargingObjects(
            { a1: createSubscription('subscriptionId', status, (new Date()).getTime(), ['a1']) }, timestamp
        )).toEqual({
            subscriptionId: {
                accounts:
                    [{
                        accountId: 'a1',
                        resources: [{ resourceId: 'cluster2', val: 0 }]
                    }],
                subscriptionId: 'subscriptionId',
                timestamp,
                dimension: CHARGING_DIMENSION
            }
        });
    });

    test('accountsToSubscriptions has one account and subscription with resources with tiering should return an object that includes the account and the subscription', async () => {
        multiTenancyOperations.getAllClustersInAccountInternal.mockReturnValue([clusterResponse]);
        const timestamp = new Date().toISOString();
        expect(await createChargingObjects(
            { a1: createSubscription('subscriptionId', status, (new Date()).getTime(), ['a1']) }, timestamp
        )).toEqual({
            subscriptionId: {
                accounts:
                    [{
                        accountId: 'a1',
                        resources: [{ resourceId: 'clusterUuid1', val: 1 }]
                    }],
                subscriptionId: 'subscriptionId',
                timestamp,
                dimension: CHARGING_DIMENSION
            }
        });
    });

    test('accountsToSubscriptions has one account and subscription with  failed resources should return an empty object', async () => {
        multiTenancyOperations.getAllClustersInAccountInternal.mockReturnValue([downClusterResponse]);
        const timestamp = new Date().toISOString();
        expect(await createChargingObjects(
            { a1: createSubscription('subscriptionId', status, (new Date()).getTime(), ['a1']) }, timestamp
        )).toEqual({});
    });

    test('accountsToSubscriptions has one account and subscription with unknown resources should return an empty object', async () => {
        multiTenancyOperations.getAllClustersInAccountInternal.mockReturnValue([unknownClusterResponse]);
        const timestamp = new Date().toISOString();
        expect(await createChargingObjects(
            { a1: createSubscription('subscriptionId', status, (new Date()).getTime(), ['a1']) }, timestamp
        )).toEqual({});
    });
    test('accountsToSubscriptions has an 2 acocunts with the same subscription , each has clusters with tieing should return an object with subscription and  accounts', async () => {
        multiTenancyOperations.getAllClustersInAccountInternal.mockReturnValueOnce([clusterResponse]).mockReturnValueOnce([clusterResponse2]);
        const timestamp = new Date().toISOString();
        expect(await createChargingObjects(
            {
                a1: createSubscription('subscriptionId',
                    status, (new Date()).getTime(),
                    ['a1', 'b1']),
                b1: createSubscription('subscriptionId',
                    status, (new Date()).getTime(),
                    ['a1', 'b1'])
            }, timestamp
        )).toEqual({
            subscriptionId: {
                accounts:
            [{
                accountId: 'a1',
                resources: [{ resourceId: 'clusterUuid1', val: 1 }]
            },
            {
                accountId: 'b1',
                resources: [{ resourceId: 'cluster2', val: 1 }]
            }],
                subscriptionId: 'subscriptionId',
                timestamp,
                dimension: CHARGING_DIMENSION
            }
        });
    });
});
