const mongoose = require('mongoose');
const auth0 = require('../../src/lib/auth0');
const { DB_CONNECTION_STRING } = require('../../src/utils/consts');
const { findWorkspace, findAccount } = require('../../src/operations/multi-tenancy-operation');
const multiTenancy = require('../../src/lib/cloud-manager/multi-tenancy');
const Account = require('../../src/models/account');
const getLogger = require('../../src/utils/logger/logger');

const logger = getLogger();

jest.mock('../../src/lib/cloud-manager/multi-tenancy.js');
jest.mock('../../src/lib/auth0');

beforeAll(async () => {
    try {
        await mongoose.connect(DB_CONNECTION_STRING, { useNewUrlParser: true, dbName: 'cloudTiering' });
    } catch (e) {
        logger.error('Cannot connect to mongoDB');
    }
});

afterAll(async done => {
    try {
        await mongoose.connection.close();
        done();
    } catch (e) {
        logger.error('Error closing connection', e);
    }
});

describe('multi-tenancy-operations', () => {
    describe('fetchOrCreateAccounts', () => {
        beforeEach(() => Account.deleteMany({}));

        test('accounts exist, return a list with account', async () => {
            multiTenancy.getAccounts.mockReturnValue([{
                userRole: 'Role-1',
                accountName: 'accountName',
                accountPublicId: '456789'
            }]);
            auth0.jwtGetUserInfo.mockReturnValue({ data: { 'http://cloud.netapp.com/user_metadata': { email: 'aa@netapp.com' } } });
            expect(await findAccount()).toEqual('456789');
        });

        test('There are no accounts, create an account and return its id', async () => {
            multiTenancy.getAccounts.mockReturnValue([]);
            multiTenancy.createAccount.mockReturnValue({
                userRole: 'Role-1',
                accountName: 'accountName',
                accountPublicId: '456789'
            });
            auth0.jwtGetUserInfo.mockReturnValue(({ data: { 'http://cloud.netapp.com/user_metadata': { email: 'aa@netapp.com' } } }));
            expect(await findAccount()).toEqual('456789');
        });
    });

    describe('fetchOrCreateWorkspaces', () => {
        const userToken = 'Bearer asdfasdf';
        const workspacePublicId = '123';
        test('Workspaces exist, return a list with workspaces', async () => {
            multiTenancy.getWorkspaces.mockReturnValue([{ workspacePublicId }]);
            expect(await findWorkspace(userToken, 'account1')).toEqual('123');
        });

        test('There are no workspaces, create an aorkspace and return its id', async () => {
            multiTenancy.getWorkspaces.mockReturnValue([]);
            multiTenancy.createWorkspace.mockReturnValue({ workspacePublicId });
            expect(await findWorkspace('userToken')).toEqual(workspacePublicId);
        });
    });
});
