const moment = require('moment');
const Account = require('../../src/models/account');
const subscriptionService = require('../../src/lib/subscription-service');
const {
    calculateDaysLeftForTrial, findSubscriptionStatus
} = require('../../src/operations/subscription-operations');
const {
    UNSUBSCRIBED, SUBSCRIBED, TIERING_SERVICE_NAME_SUBSCRIPTION
} = require('../../src/utils/consts');

const mock = jest.spyOn(Account, 'findOne'); // spy on Message.findOne()

jest.mock('../../src/lib/subscription-service');


describe('findSubscriptionStatus', () => {
    function createSubscriptionItem(subscriptionId, name, subscriptionStatus, createdAt) {
        return {
            subscriptionId,
            cloudProvider: 'aws',
            name,
            subscriptionStatus,
            serviceName: TIERING_SERVICE_NAME_SUBSCRIPTION,
            createdAt
        };
    }

    beforeEach(() => {
        subscriptionService.listSubscriptionsOfAccount.mockClear();
    });

    it(`received an empty array, should return ${UNSUBSCRIBED}`, async () => {
        subscriptionService.listSubscriptionsOfAccount.mockReturnValue([]);
        expect(await findSubscriptionStatus('accountId1')).toBe(UNSUBSCRIBED);
    });

    it(`received an array with one item- subscribed, should return subscription with status ${SUBSCRIBED}`, async () => {
        const subscription1 = createSubscriptionItem('subscriptionId5', 'customerIdentifier1', SUBSCRIBED, moment());
        subscriptionService.listSubscriptionsOfAccount.mockReturnValue([subscription1]);
        expect(await findSubscriptionStatus('accountId1')).toBe(SUBSCRIBED);
    });

    it(`received an array with one item - unsubscribed should return ${UNSUBSCRIBED}`, async () => {
        const subscription1 = createSubscriptionItem('subscriptionId6', 'customerIdentifier1', UNSUBSCRIBED, moment());
        subscriptionService.listSubscriptionsOfAccount.mockReturnValue([subscription1]);
        expect(await findSubscriptionStatus('accountId1')).toBe(UNSUBSCRIBED);
    });

    it(`received an array 2 subscription the earliest is ${SUBSCRIBED} should return him `, async () => {
        const subscription1 = createSubscriptionItem('subscriptionId', 'customerIdentifier1', UNSUBSCRIBED, moment());
        const subscription2 = createSubscriptionItem('subscriptionId2', 'customerIdentifier2', SUBSCRIBED, moment().subtract('1', 'days'));
        subscriptionService.listSubscriptionsOfAccount.mockReturnValue([
            subscription1, subscription2
        ]);
        expect(await findSubscriptionStatus('accountId1')).toBe(SUBSCRIBED);
    });
});


describe('calculateDaysLeftForTrial', () => {
    it('should return 0', async () => {
        mock.mockImplementation(() => Promise.resolve({
            name: 'account1',
            accountId: 'account1',
            freeTrialExpiryDate: moment()
        }));

        expect(await calculateDaysLeftForTrial('account1')).toEqual(0);
    });

    it('should return -1', async () => {
        mock.mockImplementation(() => Promise.resolve({
            name: 'account1',
            accountId: 'account1',
            freeTrialExpiryDate: moment().subtract(1, 'days')
        }));

        expect(await calculateDaysLeftForTrial('account1')).toEqual(-1);
    });

    it('should return a number - 3 days ', async () => {
        mock.mockImplementation(() => Promise.resolve({
            name: 'account1',
            accountId: 'account1',
            freeTrialExpiryDate: moment().add(3, 'days')
        }));

        expect(await calculateDaysLeftForTrial('account1')).toEqual(3);
    });
});
