const { URL } = require('url');
const nock = require('nock');
const Joi = require('joi');
const { isEqual, nth, get } = require('lodash');
const {
    CLOUD_MANAGER_ENDPOINT, CLOUD_MANAGER_API_PREFIX, CLOUD_MANAGER_AGGREGATES_PATH_PREFIX, CLOUD_MANAGER_AGGREGATE_ATTACH_PATH_PREFIX
} = require('../../../src/utils/consts');
const { validateSchema } = require('../../../src/utils/schema-validation');
const { escapeForwardSlash, findUserId } = require('../simulator-utils');
const { aggregateResponseSchema, objectStoreAttachRequestSchema } = require('../../../src/joi-schemas/cloud-manager-schamas');
const { createScenarioForUser, USERS } = require('../simulator-state');
const getLogger = require('../../../src/utils/logger/logger');

const logger = getLogger('simulator');

const aggregatesScope = nock(CLOUD_MANAGER_ENDPOINT)
    .log(evt => logger.info(evt))
    // get aggregates
    .get(`${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}`)
    .query(query => query.workingEnvironmentId)
    .delay(100)
    .reply(200, function (uri) {
        const userId = findUserId(this.req.headers.authorization);
        if (USERS[userId]) {
            const requestURL = new URL(uri, CLOUD_MANAGER_ENDPOINT);
            const clusterId = requestURL.searchParams.get('workingEnvironmentId');
            return JSON.stringify(USERS[userId].CLUSTERS[clusterId].aggregates);
        }
        return [404, 'User not found'];
    })

    // create aggregate
    .post(escapeForwardSlash(`${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}/.+`), body => isEqual(Object.keys(body).sort(), Object.keys(Joi.describe(aggregateResponseSchema).children).sort()))
    .delay(100)
    .reply(200, function (uri, requestBody) {
        const requestUri = new URL(uri, CLOUD_MANAGER_ENDPOINT);
        const clusterId = nth(requestUri.pathname.split('/'), 5);
        const aggregate = JSON.parse(requestBody);
        const userId = findUserId(this.req.headers.authorization);
        if (!USERS[userId]) {
            USERS[userId] = createScenarioForUser();
        }
        if (get(USERS, `${userId}.CLUSTERS.${clusterId}`)) {
            if (get(USERS, `${userId}.CLUSTERS.${clusterId}.aggregates`)) {
                USERS[userId].CLUSTERS[clusterId].aggregates.push(aggregate);
            } else {
                USERS[userId].CLUSTERS[clusterId].aggregates = [aggregate];
            }
        } else {
            return [400, 'cluster is not found'];
        }
    })
    // attach objectStore
    .post(`${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATE_ATTACH_PATH_PREFIX}`, body => {
        try {
            validateSchema(body, objectStoreAttachRequestSchema);
            return true;
        } catch (err) {
            return false;
        }
    })
    .delay(100)
    .reply(200, function (uri, requestBody) {
        const parsedBody = JSON.parse(requestBody);
        const userId = findUserId(this.req.headers.authorization);
        if (USERS[userId]) {
            const {
                workingEnvironmentId, aggregateName, objectStoreName, checkOnly
            } = parsedBody;
            if (workingEnvironmentId === 'eda35168-d036-4cce-9cc8-ca916ec36326' && aggregateName === 'aggr-3') {
                return [403, {
                    message: 'Hybrid aggregate was detected',
                    causeMessage:
                        'SimplicatorBadRequestException: aggregate not eligable for tiering'
                }];
            }
            if (USERS[userId].CLUSTERS[workingEnvironmentId]) {
                const aggregate = USERS[userId].CLUSTERS[workingEnvironmentId].aggregates.find(aggr => aggr.name === aggregateName);
                if (!aggregate) {
                    return [404, {
                        message: `Aggregate ${aggregateName} was not found`,
                        causeMessage:
                            'SimplicatorBadRequestException: Aggregate was not found'
                    }];
                }
                if (!checkOnly) {
                    aggregate.objectStoreName = objectStoreName;
                }
                aggregate.isObjectStoreAttachEligible = false;
                return [200];
            }
            return [404, 'cluster not found'];
        }
        return [404, 'User not found'];
    });

aggregatesScope.persist(true);
