const { URL } = require('url');
const uuidv4 = require('uuid/v4');
const nock = require('nock');
const {
    last, nth, flatten, compact
} = require('lodash');
const {
    MULTI_TENANCY_ENDPOINT, API_PATH_ACCOUNTS, API_PATH_RESOURCES, DISCOVER_LOCATION, API_PATH_RESOURCES_INTERNAL
} = require('../../../src/utils/consts');
const { escapeForwardSlash, findUserId } = require('../simulator-utils');
const getLogger = require('../../../src/utils/logger/logger');


const logger = getLogger('simulator');

const { USERS, createScenarioForUser } = require('../simulator-state');

const multiTenancyScope = nock(MULTI_TENANCY_ENDPOINT)
    .log(evt => logger.info(evt))
    // get accounts
    .get(API_PATH_ACCOUNTS)
    .delay(100)
    .reply(200, function () {
        const userId = findUserId(this.req.headers['x-user-token']);
        const createEmptyScenario = this.req.headers['empty-scenario'];
        if (!USERS[userId]) {
            USERS[userId] = createEmptyScenario ? {
                ACCOUNTS: { },
                CLUSTERS: {},
                AGENTS: {},
                SUBSCRIPTIONS: {}
            } : createScenarioForUser();
        }
        const accounts = [];
        Object.keys(USERS[userId].ACCOUNTS).forEach(accountKey => accounts.push(USERS[userId].ACCOUNTS[accountKey].account));
        return JSON.stringify(accounts);
    })

// create workspace
    .post(escapeForwardSlash(`${API_PATH_ACCOUNTS}/.+/.+`))
    .delay(100)
    .reply(function (uri) {
        const requestUri = new URL(uri, MULTI_TENANCY_ENDPOINT);
        const accountId = nth(requestUri.pathname.split('/'), 4);
        const userId = findUserId(this.req.headers['x-user-token']);
        if (USERS[userId] && USERS[userId].ACCOUNTS[accountId]) {
            const workspaceName = nth(requestUri.pathname.split('/'), 5);
            const workspace = {
                workspacePublicId: uuidv4(),
                workspaceName
            };

            if (USERS[userId].ACCOUNTS[accountId].workspaces) {
                USERS[userId].ACCOUNTS[accountId].workspaces.push(workspace);
            } else {
                USERS[userId].ACCOUNTS[accountId].workspaces = [workspace];
            }
            return [200, workspace];
        }
        return [404, 'No account found'];
    })

    // create account
    .post(escapeForwardSlash(`${API_PATH_ACCOUNTS}/.+`))
    .delay(100)
    .reply(200, function (uri) {
        const userId = findUserId(this.req.headers['x-user-token']);
        const createEmptyScenario = this.req.headers['empty-scenario'];
        if (!USERS[userId]) {
            USERS[userId] = createEmptyScenario ? {
                ACCOUNTS: { },
                CLUSTERS: {},
                AGENTS: {},
                SUBSCRIPTIONS: {}
            } : createScenarioForUser();
        }
        const requestUri = new URL(uri, MULTI_TENANCY_ENDPOINT);
        const account = {
            accountName: last(requestUri.pathname.split('/')),
            accountPublicId: uuidv4()
        };
        if (USERS[userId].ACCOUNTS) {
            USERS[userId].ACCOUNTS[account.accountPublicId] = { account, workspaces: [] };
        } else {
            USERS[userId].ACCOUNTS = { [account.accountPublicId]: { account, workspaces: [] } };
        }
        return { accountPublicId: account.accountPublicId };
    })


    // get workspaces
    .get(escapeForwardSlash(`${API_PATH_ACCOUNTS}/.+/workspace`))
    .delay(100)
    .reply(200, function (uri) {
        const requestUri = new URL(uri, MULTI_TENANCY_ENDPOINT);
        const accountId = nth(requestUri.pathname.split('/'), 4);
        const userId = findUserId(this.req.headers['x-user-token']);
        if (USERS[userId].ACCOUNTS[accountId]) {
            return JSON.stringify(USERS[userId].ACCOUNTS[accountId].workspaces ? USERS[userId].ACCOUNTS[accountId].workspaces : []);
        }
        return [404, 'account not found'];
    })

    // get resources
    .get(API_PATH_RESOURCES)
    .query(query => query['resource-type'] && query.account)
    .delay(100)
    .reply(200, function (uri) {
        const requestURL = new URL(uri, MULTI_TENANCY_ENDPOINT);
        const tenantId = requestURL.searchParams.get('workspace');
        const accountId = requestURL.searchParams.get('account');
        const userId = findUserId(this.req.headers['x-user-token']);
        if (USERS[userId] && USERS[userId].ACCOUNTS[accountId].workspaces) {
            if (tenantId) {
                if (USERS[userId].ACCOUNTS[accountId].workspaces.find(workspace => workspace.workspacePublicId === tenantId)) {
                    const clusters = Object.values(USERS[userId].CLUSTERS).filter(data => data.workspaceId === tenantId);
                    const shouldFail = clusters.some(cluster => cluster.cluster.ontapClusterProperties.clusterAddress === '100.100.100.100');
                    if (shouldFail) {
                        return [504, 'timoutException'];
                    }
                    return JSON.stringify(clusters.map(cluster => ({
                        resourceIdentifier: cluster.cluster.publicId,
                        agentId: cluster.agentId,
                        resourceType: DISCOVER_LOCATION
                    })));
                }
                return [404];
            }
            const workspaces = USERS[userId].ACCOUNTS[accountId].workspaces.map(workspace => workspace.workspacePublicId);
            const clusters = Object.values(USERS[userId].CLUSTERS).filter(data => workspaces.includes(data.workspaceId));
            const shouldFail = clusters.some(cluster => cluster.cluster.ontapClusterProperties.clusterAddress === '100.100.100.100');
            if (shouldFail) {
                return [504, 'timoutException'];
            }
            return JSON.stringify(clusters.map(cluster => ({
                resourceIdentifier: cluster.cluster.publicId,
                agentId: cluster.agentId,
                resourceType: DISCOVER_LOCATION
            })));
        }
        return [404];
    })
    // get resources
    .get(API_PATH_RESOURCES_INTERNAL)
    .query(query => query['resource-type'] && query.account)
    .delay(100)
    .reply(200, uri => {
        const requestURL = new URL(uri, MULTI_TENANCY_ENDPOINT);
        const accountId = requestURL.searchParams.get('account');
        const allUsersWithAccounts = Object.values(USERS).filter(user => Object.keys(user.ACCOUNTS).includes(accountId));
        const resourcesOfAccount = flatten(allUsersWithAccounts.map(user => Object.values(user.CLUSTERS)));
        return JSON.stringify(compact(resourcesOfAccount.map(cluster => {
            if (cluster.agentId) {
                return {
                    resourceIdentifier: cluster.cluster.publicId,
                    agentId: cluster.agentId,
                    resourceType: DISCOVER_LOCATION
                };
            }
            return undefined;
        })));
    });


multiTenancyScope.persist(true);
