const { URL } = require('url');
const nock = require('nock');
const { last, flatten } = require('lodash');
const { validateSchema } = require('../../../src/utils/schema-validation');
const { subscriptionSchema } = require('../../../src/joi-schemas/subscription-schemas');
const {
    SUBSCRIPTION_ENDPOINT, SUBSCRIPTION_API_PREFIX
} = require('../../../src/utils/consts');
const { escapeForwardSlash, findUserId } = require('../simulator-utils');
const { createScenarioForUser, USERS } = require('../simulator-state');
const getLogger = require('../../../src/utils/logger/logger');

const logger = getLogger('simulator');

const subscriptionServiceScope = nock(SUBSCRIPTION_ENDPOINT)
    .log(evt => logger.info(evt))
    // create subscription - fake API for testing
    .post(escapeForwardSlash(`${SUBSCRIPTION_API_PREFIX}/.+`),
        body => {
            try {
                validateSchema(body, subscriptionSchema);
                return true;
            } catch (err) {
                return false;
            }
        })
    .delay(100)
    .reply(200, function (uri, requestBody) {
        const userId = findUserId(this.req.headers.authorization);
        const requestURL = new URL(uri, SUBSCRIPTION_ENDPOINT);
        const accountId = last(requestURL.pathname.split('/'));
        const subscription = JSON.parse(requestBody);
        const { subscriptionId, subscriptionStatus } = subscription;
        if (!USERS[userId]) {
            USERS[userId] = createScenarioForUser();
        }
        // verify there is an account
        if (!USERS[userId].ACCOUNTS[accountId]) {
            return [404, 'account not found'];
        }
        if (USERS[userId].SUBSCRIPTIONS.length) {
            const subscriptionFromSubscriptions = USERS[userId].SUBSCRIPTIONS.find(sub => sub.subscriptionId === subscriptionId);
            if (subscriptionFromSubscriptions) {
                const { accounts } = subscriptionFromSubscriptions;
                if (accounts && !accounts.find(account => account === accountId)) {
                    subscriptionFromSubscriptions.accounts.push(accountId);
                } else {
                    subscriptionFromSubscriptions.accounts = [accountId];
                }
                subscriptionFromSubscriptions.subscriptionStatus = subscriptionStatus;
            } else {
                subscription.accounts = [accountId];
                USERS[userId].SUBSCRIPTIONS.push(subscription);
            }
        } else {
            subscription.accounts = [accountId];
            USERS[userId].SUBSCRIPTIONS = [subscription];
        }
        return [200, subscription];
    })

    // list subscriptions by service name
    .get(escapeForwardSlash(`${SUBSCRIPTION_API_PREFIX}/by-service-name`))
    .query(query => query.serviceName)
    .delay(100)
    .reply(uri => {
        const requestURL = new URL(uri, SUBSCRIPTION_ENDPOINT);
        const serviceName = requestURL.searchParams.get('serviceName');
        const allSubscriptions = flatten(Object.values(USERS).map(user => user.SUBSCRIPTIONS));
        const subscriptionsOfAccount = allSubscriptions.filter(subscription => subscription.serviceName === serviceName);
        return [200, subscriptionsOfAccount];
    })

    // list subscriptions by account
    .get(escapeForwardSlash(`${SUBSCRIPTION_API_PREFIX}/.+`))
    .delay(100)
    .reply(function (uri) {
        const requestURL = new URL(uri, SUBSCRIPTION_ENDPOINT);
        const accountId = last(requestURL.pathname.split('/'));
        const userId = findUserId(this.req.headers['x-user-token']);
        if (!USERS[userId]) {
            USERS[userId] = createScenarioForUser();
        }
        if (USERS[userId].ACCOUNTS[accountId]) {
            const subscriptions = Object.values(USERS[userId].SUBSCRIPTIONS).filter(subscription => subscription.accounts && subscription.accounts.includes(accountId));
            return [200, subscriptions];
        }
        return [404, 'Account was not found'];
    });
subscriptionServiceScope.persist(true);
