const { cloneDeep, isEmpty } = require('lodash');
const randomize = require('randomatic');
const moment = require('moment');
const uuidv4 = require('uuid/v4');
const getLogger = require('../../src/utils/logger/logger');
const agents = require('./resources/agents');
const clusters = require('./resources/clusters');
const aggregates = require('./resources/aggregates');
const volumes = require('./resources/volumes');
const workspaces = require('./resources/workspaces');
const mapping = require('./resources/mapping');
const licenseObj = require('./resources/license');

const logger = getLogger('simulator');

const MAPPING = {};


const USERS = {};

setInterval(() => logger.info('USERS_STATE', JSON.stringify(USERS, null, 4)), 30 * 1000);

function createAccount(trial) {
    const accountPublicId = `simulator-${randomize('Aa0', 8)}`;
    let freeTrialExpiryDate;
    if (trial === -1) {
        freeTrialExpiryDate = new Date('2/1/2016 00:00').valueOf();
    } else if (trial > 0) {
        freeTrialExpiryDate = new Date(Date.now() + moment.duration(trial, 'days')
            .as('milliseconds'));
    }
    return {
        accountName: 'simulator-account',
        accountPublicId,
        freeTrialExpiryDate
    };
}

function createScenarioForUser(mappingObj = MAPPING) {
    const scenarioMapping = mappingObj;
    if (isEmpty(scenarioMapping)) {
        Object.assign(scenarioMapping, { clusters: [], agents: [] });
    }
    // for each cluster - volumes and aggregates should be cloned for each cluster
    const volumesCollection = cloneDeep(volumes);
    const aggregatesCollection = cloneDeep(aggregates);
    const clustersCollection = cloneDeep(clusters);
    const clustersObject = scenarioMapping.clusters.reduce((accumulator, cluster) => {
        const volumesForCluster = [];
        const aggregatesForCluster = [];
        const {
            clusterId, volumes: volumesToAdd, aggregates: aggregatesToAdd, agentId, workspaceId, licenseSerial
        } = cluster;
        volumesCollection.forEach(volume => {
            if (volumesToAdd.includes(volume.name)) {
                volumesForCluster.push(cloneDeep(volume));
            }
        });
        aggregatesCollection.forEach(aggregate => {
            if (aggregatesToAdd.includes(aggregate.name)) {
                aggregatesForCluster.push(cloneDeep(aggregate));
            }
        });
        // change publicId to be dynamic
        const publicId = uuidv4();
        clustersCollection[clusterId].publicId = publicId;
        accumulator[publicId] = {
            cluster: clustersCollection[clusterId],
            volumes: volumesForCluster,
            aggregates: aggregatesForCluster,
            agentId,
            workspaceId,
            license: licenseSerial ? cloneDeep(licenseObj) : undefined
        };
        return accumulator;
    }, {});
    const agentsArray = agents.filter(agent => mappingObj.agents.includes(agent.name));
    const account = createAccount(scenarioMapping.trial);

    return {
        ACCOUNTS: { [account.accountPublicId]: { account, workspaces } },
        CLUSTERS: clustersObject,
        AGENTS: { [account.accountPublicId]: agentsArray },
        SUBSCRIPTIONS: {}
    };
}

function reset(key, userId) {
    if (key) {
        if (userId) {
            logger.info('DELETING key', key, 'For user', userId);
            Object.keys(USERS[userId][key]).forEach(subKey => delete USERS[userId][key][subKey]);
        } else {
            logger.info('DELETING key', key, 'For all users');
            Object.keys(USERS).forEach(user => {
                Object.keys(USERS[user][key]).forEach(subKey => delete USERS[user][key][subKey]);
            });
        }
    } else if (userId) {
        logger.info('DELETING user data from USERS_STATE');
        USERS[userId] = {};
    } else {
        logger.info('DELETING all data for all users');
        Object.keys(USERS).forEach(user => delete USERS[user]);
    }
}

function loadScenario(scenarioName, trial) {
    reset();
    Object.assign(MAPPING, cloneDeep(mapping[scenarioName || 'default']));
    if (trial) {
        MAPPING.trial = trial;
    }
}

function loadScenrioForUser(scenarioName, userId, trial) {
    logger.info('Loading scenario', scenarioName, 'for user', userId);
    try {
        const mappingForUser = cloneDeep(mapping[scenarioName || 'default']);
        if (trial) {
            mappingForUser.trial = trial;
        }
        reset(undefined, userId);
        USERS[userId] = createScenarioForUser(mappingForUser);
    } catch (err) {
        logger.error('Failed to load scenario', err);
    }
}

// reset user to default state - using by simulator
function resetStateForUser(userId) {
    logger.info(`reset state for userId ${userId}`);
    USERS[userId] = createScenarioForUser();
}


module.exports = {
    USERS, loadScenario, reset, loadScenrioForUser, resetStateForUser, createScenarioForUser
};
