const axios = require('axios');
const uuidv4 = require('uuid/v4');
const { reset } = require('../simulator-state');
const {
    CLOUD_MANAGER_ENDPOINT, CLOUD_MANAGER_API_PREFIX, CLOUD_MANAGER_AGGREGATES_PATH_PREFIX, CLOUD_MANAGER_AGGREGATE_ATTACH_PATH_PREFIX, CLOUD_MANAGER_CLUSTERS_PATH_PREFIX
} = require('../../../src/utils/consts');
const { signJwtToken } = require('../simulator-utils');

require('../scopes/clusters-scope');
require('../scopes/aggregates-scope');


const aggregate = {
    name: 'aggr1',
    state: 'up',
    objectStoreName: 'object-store-1',
    isInactiveDataReportingEnabled: false,
    isObjectStoreAttachEligible: true,
    usedCapacity: {
        size: 0,
        unit: 'TB'
    },
    tieringRestriction: {
        isTieringRestricted: false,
        restrictionReason: null
    }
};

const aggregate2 = {
    name: 'aggr2',
    state: 'up',
    objectStoreName: null,
    isInactiveDataReportingEnabled: false,
    isObjectStoreAttachEligible: true,
    usedCapacity: {
        size: 0,
        unit: 'TB'
    },
    tieringRestriction: {
        isTieringRestricted: false,
        restrictionReason: null
    }
};


describe('aggregate scope', () => {
    beforeEach(() => reset());

    function createCluster() {
        return {
            name: uuidv4(),
            publicId: uuidv4(),
            ontapClusterProperties: {
                clusterUuid: uuidv4(),
                ontapVersion: '9.4',
                clusterAddress: '192.168.0.1',
                nodeModels: ['1.1']
            },
            capacityFeatures: {
                objectStoreConfigExtended: {
                    objectStoreConfig: {
                        objectStoreName: 'objectStoreConfig-1',
                        s3Name: 'bucket',
                        providerType: 'AWS-S3',
                        usedSpace: {
                            size: 20,
                            unit: 'GB'
                        },
                        server: 's3.amazonaws.com'
                    },
                    tierLevel: 'normal'
                },
                performanceTierUsedCapacity: {
                    size: 1004,
                    unit: 'GB'
                },
                rawCapacity: {
                    size: 5,
                    unit: 'GB'
                },
                inactiveUserData: {
                    size: 20,
                    unit: 'GB'
                }
            }
        };
    }


    test('create/get single aggregate', async () => {
        // create cluster
        const cluster = createCluster();

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
            headers: {
                authorization: signJwtToken()
            }
        });

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}/${cluster.publicId}`, aggregate, {
            headers: {
                authorization: signJwtToken()
            }
        });

        const { data } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}?workingEnvironmentId=${cluster.publicId}&fields=tieringProperties,usedCapacity`, {
            headers: {
                authorization: signJwtToken()
            }
        });

        expect(data).toEqual([aggregate]);
    });

    test('create/get single aggregate with different users', async () => {
        // create cluster
        const cluster1 = createCluster();
        const cluster2 = createCluster();

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster1, {
            headers: {
                authorization: signJwtToken()
            }
        });


        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}/${cluster1.publicId}`, aggregate, {
            headers: {
                authorization: signJwtToken()
            }
        });


        const { data: aggregatesUser1 } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}?workingEnvironmentId=${cluster1.publicId}&fields=tieringProperties,usedCapacity`, {
            headers: {
                authorization: signJwtToken()
            }
        });

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster2, {
            headers: {
                authorization: signJwtToken('user2')
            }
        });

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}/${cluster2.publicId}`, aggregate, {
            headers: {
                authorization: signJwtToken('user2')
            }
        });

        const { data: aggregatesUser2 } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}?workingEnvironmentId=${cluster2.publicId}&fields=tieringProperties,usedCapacity`, {
            headers: {
                authorization: signJwtToken('user2')
            }
        });

        expect(aggregatesUser1).toEqual([aggregate]);
        expect(aggregatesUser2).toEqual([aggregate]);
    });

    test('create/get multiple aggregates', async () => {
        const cluster = createCluster();

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
            headers: {
                authorization: signJwtToken()
            }
        });

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}/${cluster.publicId}`, aggregate, {
            headers: {
                authorization: signJwtToken()
            }
        });

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}/${cluster.publicId}`, aggregate2, {
            headers: {
                authorization: signJwtToken()
            }
        });
        const { data } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}?workingEnvironmentId=${cluster.publicId}&fields=tieringProperties,usedCapacity`, {
            headers: {
                authorization: signJwtToken()
            }
        });

        expect(data).toEqual([aggregate, aggregate2]);
    });

    test('create/get multiple aggregates with different users', async () => {
        const cluster = createCluster();


        // create cluster

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
            headers: {
                authorization: signJwtToken()
            }
        });


        const cluster2 = createCluster();


        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}/${cluster.publicId}`, aggregate, {
            headers: {
                authorization: signJwtToken()
            }
        });

        const { data: aggreatesUser1 } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}?workingEnvironmentId=${cluster.publicId}&fields=tieringProperties,usedCapacity`, {
            headers: {
                authorization: signJwtToken()
            }
        });


        // create cluster

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster2, {
            headers: {
                authorization: signJwtToken('user2')
            }
        });
        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}/${cluster2.publicId}`, aggregate, {
            headers: {
                authorization: signJwtToken('user2')
            }
        });

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}/${cluster2.publicId}`, aggregate2, {
            headers: {
                authorization: signJwtToken('user2')
            }
        });


        const { data: aggreatesUser2 } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}?workingEnvironmentId=${cluster2.publicId}&fields=tieringProperties,usedCapacity`, {
            headers: {
                authorization: signJwtToken('user2')
            }
        });

        expect(aggreatesUser1).toEqual([aggregate]);
        expect(aggreatesUser2).toEqual([aggregate, aggregate2]);
    });


    test('attache object store - check only = true', async () => {
        const cluster = createCluster();

        // create cluster

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
            headers: {
                authorization: signJwtToken()
            }
        });
        const bodyRequest = {
            workingEnvironmentId: cluster.publicId,
            objectStoreName: 'objectStore',
            aggregateName: 'aggr1',
            checkOnly: true
        };

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}/${cluster.publicId}`, aggregate, {
            headers: {
                authorization: signJwtToken()
            }
        });

        const response = await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATE_ATTACH_PATH_PREFIX}`, bodyRequest, {
            headers: {
                authorization: signJwtToken()
            }
        });
        expect(response.status).toEqual(200);
    });

    test('attache object store to an aggregate of different user- should fail', async () => {
        try {
            const cluster = createCluster();
            // create cluster

            await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
                headers: {
                    authorization: signJwtToken()
                }
            });
            const bodyRequest = {
                workingEnvironmentId: cluster.publicId,
                objectStoreName: 'objectStore',
                aggregateName: 'aggr1',
                checkOnly: true
            };

            await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATE_ATTACH_PATH_PREFIX}`, bodyRequest, {
                headers: {
                    authorization: signJwtToken('user2')
                }
            });
        } catch (err) {
            expect(err.response.status).toEqual(404);
        }
    });

    test('attache object store - check only = false, check an objectStoreName field exists', async () => {
        const cluster = createCluster();

        // create cluster

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
            headers: {
                authorization: signJwtToken()
            }
        });

        const bodyRequest = {
            workingEnvironmentId: cluster.publicId,
            objectStoreName: 'objectStore',
            aggregateName: 'aggr2',
            checkOnly: false
        };

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}/${cluster.publicId}`, aggregate2, {
            headers: {
                authorization: signJwtToken()
            }
        });

        const response = await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATE_ATTACH_PATH_PREFIX}`, bodyRequest, {
            headers: {
                authorization: signJwtToken()
            }
        });
        expect(response.status).toEqual(200);

        const { data: aggreates } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}?workingEnvironmentId=${cluster.publicId}&fields=tieringProperties,usedCapacity`, {
            headers: {
                authorization: signJwtToken()
            }
        });

        expect(aggreates[0].objectStoreName).toEqual('objectStore');
    });
});
