const { cloneDeep } = require('lodash');
const axios = require('axios');
const uuidv4 = require('uuid/v4');
const { reset } = require('../simulator-state');
const {
    CLOUD_MANAGER_ENDPOINT, CLOUD_MANAGER_API_PREFIX, CLOUD_MANAGER_CLUSTERS_PATH_PREFIX, CLOUD_MANAGER_CLUSTER_METADATA_PATH_PREFIX,
    CLOUD_MANAGER_CONFIGURE_FP_RESOURCES_PATH_SUFFIX, CLOUD_MANAGER_INACTIVE_DATA_REPORTING_PATH_SUFFIX,
    CLOUD_MANAGER_FABRIC_POOL_LICENSE_PATH_SUFFIX, CLOUD_MANAGER_CONFIGURE_AZURE_FP_RESOURCES_PATH_SUFFIX
} = require('../../../src/utils/consts');
const { signJwtToken } = require('../simulator-utils');

require('../scopes/clusters-scope');

const UUID_REGEX = /^[\d|\w]{8}-[\d\w]{4}-[\d\w]{4}-[\d\w]{4}-[\d\w]{12}$/;

const cluster = {
    name: uuidv4(),
    publicId: uuidv4(),
    ontapClusterProperties: {
        clusterUuid: uuidv4(),
        ontapVersion: '9.4',
        clusterAddress: '192.168.0.1',
        nodeModels: ['1.1']
    },
    capacityFeatures: {
        objectStoreConfigExtended: {
            objectStoreConfig: {
                objectStoreName: 'objectStoreConfig-1',
                s3Name: 'bucket',
                providerType: 'AWS-S3',
                usedSpace: {
                    size: 20,
                    unit: 'GB'
                },
                server: 's3.amazonaws.com'
            },
            tierLevel: 'normal'
        },
        performanceTierUsedCapacity: {
            size: 1004,
            unit: 'GB'
        },
        rawCapacity: {
            size: 5,
            unit: 'GB'
        },
        inactiveUserData: {
            size: 20,
            unit: 'GB'
        }
    }
};

const license = {
    statusResp: {
        serialNumber: '123456',
        version: '1',
        product: 'fabricPool',
        licenses: {
            capacity: '10',
            type: 'capacity',
            package: 'fabricPool'
        }
    },
    trackingId: '123',
    Signature: 'signature'
};

describe('cloud manager scope', () => {
    beforeEach(() => reset('CLUSTERS'));

    test('create single cluster', async () => {
        const { data } = await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
            headers: {
                authorization: signJwtToken()
            }
        });

        expect(data).toMatchObject(cluster);
    });

    test('metadata existing cluster', async () => {
        const { data: { ontapClusterProperties: { clusterAddress } } } = await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
            headers: {
                authorization: signJwtToken()
            }
        });

        const { data } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTER_METADATA_PATH_PREFIX}`, {
            headers: {
                clusterAddress,
                authorization: signJwtToken()

            }
        });

        expect(data).toMatchObject({
            clusterName: expect.stringMatching(UUID_REGEX),
            ontapVersion: '9.4',
            nodeModels: ['1.1']
        });
    });

    test('metadata non existing cluster', async () => {
        try {
            await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTER_METADATA_PATH_PREFIX}`, {
                headers: {
                    clusterAddress: 'no such cluster',
                    authorization: signJwtToken()
                }
            });
        } catch (err) {
            expect(err.response.status).toBe(404);
        }
    });

    test('discover existing cluster', async () => {
        const { data: { ontapClusterProperties: { clusterAddress } } } = await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
            headers: {
                authorization: signJwtToken()
            }
        });

        const { data } = await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, {
            tenantId: 'workspace-1',
            name: 'cluster-1',
            clusterAddress,
            clusterUserName: 'user',
            clusterPassword: 'secret',
            location: 'ON_PREM'
        }, {
            headers: {
                'X-Agent-Id': 'agent-1',
                authorization: signJwtToken()
            }
        });

        expect(data).toMatchObject({
            publicId: expect.stringMatching(UUID_REGEX)
        });
    });

    test('try to discover a cluster of a different user-should fail', async () => {
        try {
            const { data: { ontapClusterProperties: { clusterAddress } } } = await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
                headers: {
                    authorization: signJwtToken()
                }
            });

            await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, {
                tenantId: 'workspace-1',
                name: 'cluster-1',
                clusterAddress,
                clusterUserName: 'user',
                clusterPassword: 'secret',
                location: 'ON_PREM'
            }, {
                headers: {
                    'X-Agent-Id': 'agent-1',
                    authorization: signJwtToken('user2')
                }
            });
        } catch (err) {
            expect(err.response.status).toBe(404);
        }
    });

    test('discover non existing cluster', async () => {
        try {
            await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, {
                tenantId: 'workspace-1',
                name: 'cluster-1',
                clusterAddress: 'no such cluster',
                clusterUserName: 'user',
                clusterPassword: 'secret',
                location: 'ON_PREM'
            }, {
                headers: {
                    'X-Agent-Id': 'agent-1',
                    authorization: signJwtToken()
                }
            });
        } catch (err) {
            expect(err.response.status).toBe(404);
        }
    });

    test('cluster info - existing cluster', async () => {
        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
            headers: {
                authorization: signJwtToken()
            }
        });

        const { data } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${cluster.publicId}?fields=ontapClusterProperties,capacityFeatures,status`, {
            headers: {
                authorization: signJwtToken()
            }
        });

        expect(data).toMatchObject(cluster);
    });

    test('cluster info - non existing cluster', async () => {
        try {
            await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${uuidv4()}?fields=ontapClusterProperties,capacityFeatures,status`, {
                headers: {
                    authorization: signJwtToken()
                }
            });
        } catch (err) {
            expect(err.response.status).toBe(404);
        }
    });


    test('cluster-info- cluster info of a different user - should fail', async () => {
        try {
            await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
                headers: {
                    authorization: signJwtToken()
                }
            });

            await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${cluster.publicId}?fields=ontapClusterProperties,capacityFeatures,status`, {
                headers: {
                    authorization: signJwtToken('user2')
                }
            });
        } catch (err) {
            expect(err.response.status).toBe(404);
        }
    });
    test('fabric pool resources - non existing cluster', async () => {
        try {
            const body = {
                region: 'us-east-1',
                awsAccessKeys: {
                    accessKey: 'aa',
                    secretKey: 'bb'
                },
                tierLevel: 'ia'
            };
            await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/cluster/${CLOUD_MANAGER_CONFIGURE_FP_RESOURCES_PATH_SUFFIX}`, body, {
                headers: {
                    authorization: signJwtToken()
                }
            });
        } catch (err) {
            expect(err.response.status).toBe(404);
        }
    });

    test('fabric pool resources -  existing cluster', async () => {
        const body = {
            region: 'us-east-1',
            awsAccessKeys: {
                accessKey: 'aa',
                secretKey: 'bb'
            },
            tierLevel: 'ia'
        };
        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
            headers: {
                authorization: signJwtToken()
            }
        });
        const { data } = await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${cluster.publicId}/${CLOUD_MANAGER_CONFIGURE_FP_RESOURCES_PATH_SUFFIX}`, body, {
            headers: {
                authorization: signJwtToken()
            }
        });
        expect(data.objectStoreName).toBe('tiering_objectStore');
    });

    test('azure fabric pool resources - non existing cluster', async () => {
        try {
            const body = {
                region: 'centralus'
            };
            await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/cluster/${CLOUD_MANAGER_CONFIGURE_AZURE_FP_RESOURCES_PATH_SUFFIX}`, body, {
                headers: {
                    authorization: signJwtToken()
                }
            });
        } catch (err) {
            expect(err.response.status).toBe(404);
        }
    });

    test('azure fabric pool resources -  existing cluster', async () => {
        const body = {
            region: 'centralus'
        };
        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
            headers: {
                authorization: signJwtToken()
            }
        });
        const { data } = await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${cluster.publicId}/${CLOUD_MANAGER_CONFIGURE_AZURE_FP_RESOURCES_PATH_SUFFIX}`, body, {
            headers: {
                authorization: signJwtToken()
            }
        });
        expect(data.objectStoreName).toBe('tiering_objectStore');
    });


    test('enableColdDataReporting- non existing cluster', async () => {
        try {
            const body = {
                enable: true
            };
            await axios.put(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${uuidv4()}/${CLOUD_MANAGER_INACTIVE_DATA_REPORTING_PATH_SUFFIX}`, body, {
                headers: {
                    authorization: signJwtToken()
                }
            });
        } catch (err) {
            expect(err.response.status).toBe(404);
        }
    });

    test('enableColdDataReporting-   existing cluster', async () => {
        const body = {
            enable: true
        };
        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster,
            {
                headers: {
                    authorization: signJwtToken()
                }
            });
        const { status } = await axios.put(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${cluster.publicId}/${CLOUD_MANAGER_INACTIVE_DATA_REPORTING_PATH_SUFFIX}`, body, {
            headers: {
                authorization: signJwtToken()
            }
        });
        expect(status).toBe(200);
    });

    test('delete single cluster', async () => {
        const { data } = await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster,
            {
                headers: {
                    authorization: signJwtToken()
                }
            });
        expect(data).toMatchObject(cluster);
        await axios.delete(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${cluster.publicId}`, {
            headers: {
                authorization: signJwtToken()
            }
        });

        try {
            await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${cluster.publicId}?fields=ontapClusterProperties,capacityFeatures`, {
                headers: {
                    authorization: signJwtToken()
                }
            });
        } catch (err) {
            expect(err.response.status).toBe(404);
        }
    });

    test('delete a cluster of a different user- should fail', async () => {
        try {
            const { data } = await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster,
                {
                    headers: {
                        authorization: signJwtToken()
                    }
                });
            expect(data).toMatchObject(cluster);
            await axios.delete(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${cluster.publicId}`, {
                headers: {
                    authorization: signJwtToken('user2')
                }
            });
        } catch (err) {
            expect(err.response.status).toBe(404);
        }
    });


    test('delete all clusters', async () => {
        const cluster1 = cluster;
        const cluster2 = cloneDeep(cluster);
        cluster2.publicId = uuidv4();
        const res1 = await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster1,
            {
                headers: {
                    authorization: signJwtToken()
                }
            });
        const res2 = await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster2,
            {
                headers: {
                    authorization: signJwtToken()
                }
            });
        expect(res1.data).toMatchObject(cluster1);
        expect(res2.data).toMatchObject(cluster2);
        await axios.delete(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`,
            {
                headers: {
                    authorization: signJwtToken()
                }
            });

        try {
            await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${cluster1.publicId}?fields=ontapClusterProperties,capacityFeatures`,
                {
                    headers: {
                        authorization: signJwtToken()
                    }
                });
        } catch (err) {
            expect(err.response.status).toBe(404);
        }
        try {
            await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${cluster2.publicId}?fields=ontapClusterProperties,capacityFeatures`,
                {
                    headers: {
                        authorization: signJwtToken()
                    }
                });
        } catch (err) {
            expect(err.response.status).toBe(404);
        }
    });

    test('apply-license on cluster', async () => {
        // create cluster first
        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
            headers: {
                authorization: signJwtToken()
            }
        });

        const response = await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${cluster.publicId}/${CLOUD_MANAGER_FABRIC_POOL_LICENSE_PATH_SUFFIX}`, {
            statusResp: {
                serialNumber: '123456',
                version: '1',
                product: 'fabricPool',
                licenses: {
                    capacity: '10',
                    type: 'capacity',
                    package: 'fabricPool'
                }
            },
            trackingId: '123',
            Signature: 'signature'
        },
        {
            headers: { authorization: signJwtToken() }
        });
        expect(response.status).toBe(200);
    });

    test('get license on cluster', async () => {
        // create cluster first
        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
            headers: {
                authorization: signJwtToken()
            }
        });

        // apply license
        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${cluster.publicId}/${CLOUD_MANAGER_FABRIC_POOL_LICENSE_PATH_SUFFIX}`, license,
            {
                headers: { authorization: signJwtToken() }
            });

        const { data, status } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/${cluster.publicId}/${CLOUD_MANAGER_FABRIC_POOL_LICENSE_PATH_SUFFIX}`,
            {
                headers: { authorization: signJwtToken() }
            });

        expect(status).toBe(200);
        expect(data).toMatchObject(license);
    });
});
