const axios = require('axios');
const uuidv4 = require('uuid/v4');
const { reset } = require('../simulator-state');
const {
    MULTI_TENANCY_ENDPOINT, API_PATH_ACCOUNTS, CLOUD_MANAGER_ENDPOINT, CLOUD_MANAGER_API_PREFIX, CLOUD_MANAGER_CLUSTERS_PATH_PREFIX,
    API_PATH_RESOURCES, DISCOVER_LOCATION, API_PATH_RESOURCES_INTERNAL
} = require('../../../src/utils/consts');
const { signJwtToken } = require('../simulator-utils');
require('../scopes/multi-tenancy-scope');


require('../scopes/clusters-scope');

const UUID_REGEX = /^[\d|\w]{8}-[\d\w]{4}-[\d\w]{4}-[\d\w]{4}-[\d\w]{12}$/;
const ACCOUNT_1_NAME = 'account-1';
const ACCOUNT_2_NAME = 'account-2';
const WORKSPACE_1_NAME = 'workspace-1';

const WORKSPACE_2_NAME = 'workspace-2';


describe('multi tenancy scope', () => {
    beforeEach(() => reset());


    test('accounts - create single', async () => {
        const { data } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken(),
                emptyScenario: true
            }
        });
        expect(data).toMatchObject({ accountPublicId: expect.stringMatching(UUID_REGEX) });
    });

    test('accounts - get single', async () => {
        const { data: account } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken(),
                'empty-scenario': true
            }
        });
        const { data: accounts } = await axios.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken(),
                'empty-scenario': true
            }
        });

        expect(accounts).toEqual([{
            accountName: ACCOUNT_1_NAME,
            accountPublicId: account.accountPublicId
        }]);
    });

    test('accounts - get empty', async () => {
        const { data } = await axios.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken(),
                'empty-scenario': true
            }
        });

        expect(data).toEqual([]);
    });

    test('accounts - create multiple with the same user', async () => {
        const { data: account1 } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken(),
                'empty-scenario': true
            }
        });
        const { data: account2 } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_2_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken(),
                'empty-scenario': true
            }
        });
        const { data: accounts } = await axios.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken(),
                'empty-scenario': true
            }
        });
        expect(accounts).toEqual([
            {
                accountName: ACCOUNT_1_NAME,
                accountPublicId: account1.accountPublicId
            }, {
                accountName: ACCOUNT_2_NAME,
                accountPublicId: account2.accountPublicId
            }
        ]);
    });


    test('accounts - create multiple with the 2 different users', async () => {
        const { data: account1 } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken(),
                'empty-scenario': true
            }
        });
        const { data: account2 } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_2_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2'),
                'empty-scenario': true
            }
        });
        const { data: accountsUser1 } = await axios.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken(),
                'empty-scenario': true
            }
        });
        const { data: accountsUser2 } = await axios.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}`, {
            headers: {
                'X-User-Token': signJwtToken('user2'),
                authorization: signJwtToken('user2'),
                'empty-scenario': true
            }
        });
        expect(accountsUser1).toEqual([
            {
                accountName: ACCOUNT_1_NAME,
                accountPublicId: account1.accountPublicId
            }]);

        expect(accountsUser2).toEqual([{
            accountName: ACCOUNT_2_NAME,
            accountPublicId: account2.accountPublicId
        }]);
    });


    test('workspaces - create single', async () => {
        const { data: { accountPublicId } } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${accountPublicId}/${WORKSPACE_1_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        expect(data).toEqual({
            workspaceName: WORKSPACE_1_NAME,
            workspacePublicId: expect.stringMatching(UUID_REGEX)
        });
    });

    test('workspaces - create for non existing account', async () => {
        try {
            await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${uuidv4()}/${WORKSPACE_1_NAME}`, undefined, {
                headers: {
                    'X-User-Token': signJwtToken(),
                    authorization: signJwtToken()
                }
            });
        } catch (err) {
            expect(err.response.status).toBe(404);
        }
    });

    test('workspaces - get single', async () => {
        const { data: { accountPublicId } } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: workspace } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${accountPublicId}/${WORKSPACE_1_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: workspaces } = await axios.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${accountPublicId}/workspace`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        expect(workspaces).toEqual([workspace]);
    });

    test('workspaces - get non existent', async () => {
        const { data: { accountPublicId } } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${accountPublicId}/${WORKSPACE_1_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        try {
            await axios.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${uuidv4()}/workspace`, {
                headers: {
                    'X-User-Token': signJwtToken(),
                    authorization: signJwtToken()
                }
            });
        } catch (err) {
            expect(err.response.status).toEqual(404);
        }
    });

    test('workspaces - create multiple workspaces in same account', async () => {
        const { data: { accountPublicId } } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: workspace1 } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${accountPublicId}/${WORKSPACE_1_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: workspace2 } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${accountPublicId}/${WORKSPACE_2_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: workspaces } = await axios.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${accountPublicId}/workspace`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        expect(workspaces).toEqual([workspace1, workspace2]);
    });

    test('workspaces - create multiple workspaces in different accounts', async () => {
        const { data: { accountPublicId: account1PublicId } } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: { accountPublicId: account2PublicId } } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_2_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: workspace1 } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${account1PublicId}/${WORKSPACE_1_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: workspace2 } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${account2PublicId}/${WORKSPACE_2_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: workspacesAccount1 } = await axios.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${account1PublicId}/workspace`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: workspacesAccount2 } = await axios.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${account2PublicId}/workspace`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });

        expect(workspacesAccount1).toEqual([workspace1]);
        expect(workspacesAccount2).toEqual([workspace2]);
    });

    const clusterAddress = '192.168.0.1';

    const cluster = {
        name: uuidv4(),
        publicId: uuidv4(),
        ontapClusterProperties: {
            clusterUuid: uuidv4(),
            ontapVersion: '9.4',
            clusterAddress,
            nodeModels: ['1.1']
        },
        capacityFeatures: {
            objectStoreConfigExtended: {
                objectStoreConfig: {
                    objectStoreName: 'objectStoreConfig-1',
                    s3Name: 'bucket',
                    providerType: 'AWS-S3',
                    usedSpace: {
                        size: 20,
                        unit: 'GB'
                    },
                    server: 's3.amazonaws.com'
                },
                tierLevel: 'normal'
            },
            performanceTierUsedCapacity: {
                size: 1004,
                unit: 'GB'
            },
            rawCapacity: {
                size: 5,
                unit: 'GB'
            },
            inactiveUserData: {
                size: 20,
                unit: 'GB'
            }
        }
    };

    test('resources - get resources of existing workspace with one cluster', async () => {
        // create account
        // create workspace
        // create cluster
        // discover inside workspace
        const { data: { accountPublicId: account1PublicId } } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: workspace1 } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${account1PublicId}/${WORKSPACE_1_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: { publicId } } = await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, {
            tenantId: workspace1.workspacePublicId,
            name: 'cluster-1',
            clusterAddress,
            clusterUserName: 'user',
            clusterPassword: 'secret',
            location: 'ON_PREM'
        }, {
            headers: {
                'X-Agent-Id': 'agent-1',
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        //
        const { data: getResourcesResult } = await axios.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_RESOURCES}?workspace=${workspace1.workspacePublicId}&account=${account1PublicId}&resource-type=${DISCOVER_LOCATION}`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        //
        expect(getResourcesResult).toEqual([{ resourceIdentifier: publicId, agentId: 'agent-1', resourceType: DISCOVER_LOCATION }]);
    });

    test('resources - get resources of existing account with one cluster', async () => {
        // create account
        // create workspace
        // create cluster
        // discover inside workspace
        const { data: { accountPublicId: account1PublicId } } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: workspace1 } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${account1PublicId}/${WORKSPACE_1_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: { publicId } } = await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, {
            tenantId: workspace1.workspacePublicId,
            name: 'cluster-1',
            clusterAddress,
            clusterUserName: 'user',
            clusterPassword: 'secret',
            location: 'ON_PREM'
        }, {
            headers: {
                'X-Agent-Id': 'agent-1',
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        //
        const { data: getResourcesResult } = await axios.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_RESOURCES}?&account=${account1PublicId}&resource-type=${DISCOVER_LOCATION}`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        expect(getResourcesResult).toEqual([{ resourceIdentifier: publicId, agentId: 'agent-1', resourceType: DISCOVER_LOCATION }]);
    });

    test('resources - get resources of  non-existing workspace - should receive status 404', async () => {
        try {
            await axios.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_RESOURCES}?workspace=workspacePublicId&account=account1PublicId&resource-type=${DISCOVER_LOCATION}`, {
                headers: {
                    'X-User-Token': signJwtToken(),
                    authorization: signJwtToken()
                }
            });
        } catch (err) {
            expect(err.response.status).toBe(404);
        }
    });

    test('resources - internal - get resources of existing workspace with one cluster', async () => {
        // create account
        // create workspace
        // create cluster
        // discover inside workspace
        const { data: { accountPublicId: account1PublicId } } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: workspace1 } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${account1PublicId}/${WORKSPACE_1_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
            headers: {
                authorization: signJwtToken()
            }
        });
        const { data: { publicId } } = await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, {
            tenantId: workspace1.workspacePublicId,
            name: 'cluster-1',
            clusterAddress,
            clusterUserName: 'user',
            clusterPassword: 'secret',
            location: 'ON_PREM'
        }, {
            headers: {
                'X-Agent-Id': 'agent-1',
                authorization: signJwtToken()
            }
        });
        //
        const { data: getResourcesResult } = await axios.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_RESOURCES_INTERNAL}?workspace=${workspace1.workspacePublicId}&account=${account1PublicId}&resource-type=${DISCOVER_LOCATION}`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        //
        expect(getResourcesResult).toEqual([{ resourceIdentifier: publicId, agentId: 'agent-1', resourceType: DISCOVER_LOCATION }]);
    });

    test('resources - internal - get resources of existing account with one cluster', async () => {
        // create account
        // create workspace
        // create cluster
        // discover inside workspace
        const { data: { accountPublicId: account1PublicId } } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${ACCOUNT_1_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        const { data: workspace1 } = await axios.post(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${account1PublicId}/${WORKSPACE_1_NAME}`, undefined, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
            headers: {
                authorization: signJwtToken()
            }
        });
        const { data: { publicId } } = await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, {
            tenantId: workspace1.workspacePublicId,
            name: 'cluster-1',
            clusterAddress,
            clusterUserName: 'user',
            clusterPassword: 'secret',
            location: 'ON_PREM'
        }, {
            headers: {
                'X-Agent-Id': 'agent-1',
                authorization: signJwtToken()
            }
        });
        //
        const { data: getResourcesResult } = await axios.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_RESOURCES_INTERNAL}?&account=${account1PublicId}&resource-type=${DISCOVER_LOCATION}`, {
            headers: {
                'X-User-Token': signJwtToken(),
                authorization: signJwtToken()
            }
        });
        expect(getResourcesResult).toEqual([{ resourceIdentifier: publicId, agentId: 'agent-1', resourceType: DISCOVER_LOCATION }]);
    });

    test('resources - internal - get resources of  non-existing workspace - should receive status 404', async () => {
        try {
            await axios.get(`${MULTI_TENANCY_ENDPOINT}${API_PATH_RESOURCES_INTERNAL}?workspace=workspacePublicId&account=account1PublicId&resource-type=${DISCOVER_LOCATION}`, {
                headers: {
                    'X-User-Token': signJwtToken(),
                    authorization: signJwtToken()
                }
            });
        } catch (err) {
            expect(err.response.status).toBe(404);
        }
    });
});
