const axios = require('axios');
const uuidv4 = require('uuid/v4');
const { reset } = require('../simulator-state');
const {
    CLOUD_MANAGER_ENDPOINT, CLOUD_MANAGER_API_PREFIX, CLOUD_MANAGER_VOLUMES_PATH_PREFIX, CLOUD_MANAGER_CLUSTERS_PATH_PREFIX
} = require('../../../src/utils/consts');
const { signJwtToken } = require('../simulator-utils');

require('../scopes/volumes-scope');
require('../scopes/clusters-scope');


const volume = {
    name: 'volumeName',
    svmName: 'svm_1',
    aggregateName: 'aggr-1',
    size: { size: 10, unit: 'TB' },
    usedSize: { size: 5, unit: 'TB' },
    tieringPolicy: 'auto',
    constituentsAggregates: []
};

const volume2 = {
    name: 'vol2',
    svmName: 'svm_1',
    aggregateName: 'aggr-1',
    size: { size: 10, unit: 'TB' },
    usedSize: { size: 5, unit: 'TB' },
    tieringPolicy: 'auto',
    constituentsAggregates: []
};

function createCluster() {
    return {
        name: uuidv4(),
        publicId: uuidv4(),
        ontapClusterProperties: {
            clusterUuid: uuidv4(),
            ontapVersion: '9.4',
            clusterAddress: '192.168.0.1',
            nodeModels: ['1.1']
        },
        capacityFeatures: {
            objectStoreConfigExtended: {
                objectStoreConfig: {
                    objectStoreName: 'objectStoreConfig-1',
                    s3Name: 'bucket',
                    providerType: 'AWS-S3',
                    usedSpace: {
                        size: 20,
                        unit: 'GB'
                    },
                    server: 's3.amazonaws.com'
                },
                tierLevel: 'normal'
            },
            performanceTierUsedCapacity: {
                size: 1004,
                unit: 'GB'
            },
            rawCapacity: {
                size: 5,
                unit: 'GB'
            },
            inactiveUserData: {
                size: 20,
                unit: 'GB'
            }
        }
    };
}


describe('volume scope', () => {
    beforeEach(() => reset('CLUSTERS'));

    test('create volume', async () => {
        const cluster = createCluster();

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
            headers: {
                authorization: signJwtToken()
            }
        });

        const { data } = await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}/${cluster.publicId}`, volume, {
            headers: {
                authorization: signJwtToken()
            }
        });
        expect(data).toMatchObject(volume);
    });

    test('get volumes when there  are no volumes ', async () => {
        const cluster = createCluster();

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
            headers: {
                authorization: signJwtToken()
            }
        });

        const { data } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}?workingEnvironmentId=${cluster.publicId}`, {
            headers: {
                authorization: signJwtToken()
            }
        });
        expect(data).toMatchObject([]);
    });

    test('get volumes when there is a volumes ', async () => {
        const cluster = createCluster();

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
            headers: {
                authorization: signJwtToken()
            }
        });

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}/${cluster.publicId}`, volume, {
            headers: {
                authorization: signJwtToken()
            }
        });
        const { data } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}?workingEnvironmentId=${cluster.publicId}`, {
            headers: {
                authorization: signJwtToken()
            }
        });
        expect(data).toMatchObject([volume]);
    });

    test('create/get volumes of different users- each user should receive its own volumes', async () => {
        const cluster = createCluster();

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
            headers: {
                authorization: signJwtToken()
            }
        });


        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}/${cluster.publicId}`, volume, {
            headers: {
                authorization: signJwtToken()
            }
        });
        const { data: volumeUser1 } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}?workingEnvironmentId=${cluster.publicId}`, {
            headers: {
                authorization: signJwtToken()
            }
        });

        const cluster2 = createCluster();

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster2, {
            headers: {
                authorization: signJwtToken('user2')
            }
        });
        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}/${cluster2.publicId}`, volume2, {
            headers: {
                authorization: signJwtToken('user2')
            }
        });
        const { data: volumeUser2 } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}?workingEnvironmentId=${cluster2.publicId}`, {
            headers: {
                authorization: signJwtToken('user2')
            }
        });

        expect(volumeUser1).toMatchObject([volume]);
        expect(volumeUser2).toMatchObject([volume2]);
    });

    test('modify volumes - modify for non existing volume ', async () => {
        try {
            await axios.put(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}/cluster_1/svm_name/volumeName`,
                { tieringPolicy: 'auto' }, {
                    headers: {
                        authorization: signJwtToken()
                    }
                });
        } catch (err) {
            expect(err.response.status).toBe(404);
        }
    });

    test('modify volumes - modify for existing volume ', async () => {
        const cluster = createCluster();

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
            headers: {
                authorization: signJwtToken()
            }
        });

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}/${cluster.publicId}`, volume, {
            headers: {
                authorization: signJwtToken()
            }
        });
        const { data } = await axios.put(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}/${cluster.publicId}/svm_1/volumeName`,
            { tieringPolicy: 'auto' }, {
                headers: {
                    authorization: signJwtToken()
                }
            });
        expect(data.volumeName).toEqual('volumeName');
    });

    test('modify volumes- try to modify a volume of a different user', async () => {
        try {
            const cluster = createCluster();

            await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, cluster, {
                headers: {
                    authorization: signJwtToken()
                }
            });


            await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}/${cluster.publicId}`, volume, {
                headers: {
                    authorization: signJwtToken()
                }
            });

            await axios.put(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_VOLUMES_PATH_PREFIX}/${cluster.publicId}/svm_1/volumeName`,
                { tieringPolicy: 'auto' }, {
                    headers: {
                        authorization: signJwtToken()
                    }
                });
        } catch (err) {
            expect(err.response.status).toEqual(404);
        }
    });
});
